#!/bin/bash
# $Id $
#
# Wrapper script to be used for "misbehaving" programs that 
# write everyting to stderr, including important output and
# error messages. 
#
# The wrapper calls the program and redirect the output to 
# stdout if the program exits with 0 or to stderr if it
# exits with some other value.
# 
# Usage: ./stdwrap cmd [other parameters] 
#

# create temporary file in current directory
TMP=`mktemp --tmpdir=.`

# start program and redirect stderr + stdout to the temporary file 
CMD=$1
shift
${CMD} "${@}" 2> ${TMP} 1>&2 

# get exit code
EXIT_CODE=$?

if [ ${EXIT_CODE} -eq 0 ]; then
	# exit code is 0, copy temporary file to stdout
    cat ${TMP}
	rm ${TMP}
else
	# exit code is != 0, copy temporary file to stderr
    cat ${TMP} 1>&2
    rm ${TMP}
	# exit with same exit code
	exit ${EXIT_CODE}
fi

