var GenoType = function()
{
	var gt = {};
	var debug = 0;
	var selectionIsValid = false;
	var files;
	
	// Page initialization
	gt.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnBrowse', gt.selectFiles);
		Events.addEventHandler('files', 'base-selected', gt.setFileCallback);
		Events.addEventHandler('files', 'change', gt.filesOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', gt.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', gt.initializeStep2);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		//Events.addEventHandler('wizard', 'wizard-submit', gt.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=PrepareForGenotypeChecks';
		Wizard.asyncJsonRequest(url, gt.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	gt.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];	

		var msg = response.existingAlignmentsCount + ' alignments';
		if (response.existingGenotypeCallsCount) msg += ' and '+response.existingGenotypeCallsCount+' genotype calls';
		Doc.element('alreadyChecked').innerHTML = msg;

		Doc.show('step-1');
		Doc.show('gonext');

		frm.files.focus();
	}
	
	gt.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}
	
	gt.selectFiles = function()
	{
		var frm = document.forms['reggie'];
		if (frm.files.disabled) return;
		
		var url = '&resetTemporary=1';
		url += '&tmpfilter:STRING:name='+encodeURIComponent('%.vcf|%.vcf.gz');
		Dialogs.selectItem('FILE', 'files', 1, url);
	}

	gt.setFileCallback = function(event)
	{
		var frm = document.forms['reggie'];
		var file = event.detail;
		App.debug(event.detail);
		var opt = Reggie.getListOption('files', file.id);
		if (opt)
		{
			opt.selected = true;
		}
		else
		{
			var option = gt.createListOption(frm.files.length+1, file, true);
			frm.files[frm.files.length] = option;
		}
		
		if (file.remaining == 0) 
		{
			Events.sendChangeEvent('files');
			frm.files.focus();
		}
	}

	gt.createListOption = function(index, file, selected)
	{
		var name = index+': '+file.name;
		var option = new Option(name, file.id, false, selected);
		option.file = file;
		return option;	
	}
	
	gt.filesOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		selectionIsValid = false;
		var numSelected = 0;
		Wizard.setInputStatus('files');
		Wizard.hideGoNextConfirmation();

		for (var fNo = 0; fNo < frm.files.length; fNo++)
		{
			if (frm.files[fNo].selected) 
			{
				numSelected++;
			}
		}
		
		if (numSelected == 0)
		{
			Wizard.setInputStatus('files', 'invalid', 'Select at least one file');
			return;
		}
		selectionIsValid = true;
		Wizard.setInputStatus('files', 'valid');
	}

	gt.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		
		var submitInfo = {};
		var files = [];
		submitInfo.files = files;
			
		for (var fNo = 0; fNo < frm.files.length; fNo++)
		{
			if (frm.files[fNo].selected) 
			{
				files[files.length] = { 'id': parseInt(frm.files[fNo].file.id) };
			}
		}

		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=GenotypeStandaloneFileCheck';
		Wizard.showLoadingAnimation('Comparing genotypes...', 'genotypes-progress');
		Wizard.asyncJsonRequest(url, gt.checkCompleted, 'POST', JSON.stringify(submitInfo));
	}
	
	gt.checkCompleted = function(response)
	{
		files = response.files;
		
		var html = '<table id="compareTable">';
		html += '<thead class="bg-filled-100">';
		html += '<tr>';
		html += '<th colspan="3">Comparisons</th>';
		html += '<th class="dottedleft" colspan="5">Genotypes</th>';
		html += '<th class="dottedleft" colspan="5"></th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		html += '<tr>';
		html += '<th>Total</th>';
		html += '<th>Same PAT</th>';
		html += '<th>Skipped</th>';
		html += '<th class="dottedleft">File</th>';
		html += '<th>Total</th>';
		html += '<th>Usable</th>';
		html += '<th>HET</th>';
		html += '<th class="icon-col"></th>';
		html += '<th class="dottedleft icon-col"></th>';
		html += '<th colspan="3">Warnings and messages</th>';
		html += '<th class="icon-col"></th>';
		html += '<th class="dottedleft comment"></th>';
		html += '</tr>';
		html += '</thead>';
		
		var htmlNoMessages = '';
		var htmlNoWarnings = '';
		
		for (var fNo = 0; fNo < files.length; fNo++)
		{
			var file = files[fNo];
			var cmp = file.compare;
			var vcf = cmp.vcf;
			
			var hetPercentage = vcf.gtCount ? Reggie.formatNumber(100*vcf.hetCount/vcf.gtCount, '%', -1) : '-';
			var rowspan = cmp.messages.length || 1;
			
			var tmp = '<tbody class="highlight"><tr>';
			tmp += '<td rowspan="'+rowspan+'">'+cmp.total+'</td>';
			tmp += '<td rowspan="'+rowspan+'">'+(cmp.samePat || '')+'</td>';
			tmp += '<td rowspan="'+rowspan+'">'+(cmp.skipped || '')+'</td>';
			tmp += '<td rowspan="'+rowspan+'" class="dottedleft text-col">'+Strings.encodeTags(file.path)+'</td>';
			tmp += '<td rowspan="'+rowspan+'">'+vcf.totalCount+'</td>';
			tmp += '<td rowspan="'+rowspan+'">'+vcf.gtCount+'</td>';
			tmp += '<td rowspan="'+rowspan+'">'+hetPercentage+'</td>';
			tmp += '<td rowspan="'+rowspan+'" class="icon-col">';
			tmp += '<span class="link vcf-link" data-file-id="'+vcf.fileId+'" title="View genotype statistics"><img src="../images/vcf_file.png"></span>';
			tmp += '</td>';

			// The first message is on the main <tr>
			var numWarnings = 0;
			if (cmp.messages.length > 0)
			{
				var msg = cmp.messages[0];
				if (msg.warningLevel != 'INFO') numWarnings++;
				tmp += gt.getMessageHtml(msg, file, vcf);
			}
			else
			{
				tmp += '<td class="dottedleft icon-col"></td>';
				tmp += '<td class="text-col" colspan="2"></td>';
				tmp += '<td class="icon-col"></td>';
			}
			tmp += '<td rowspan="'+rowspan+'" class="dottedleft comment"></td>';
			tmp += '</tr>';
			
			// If there are more messages, add more <tr>
			for (var mNo = 1; mNo < cmp.messages.length; mNo++)
			{
				var msg = cmp.messages[mNo];
				if (msg.warningLevel != 'INFO') numWarnings++;
				tmp += '<tr>'+gt.getMessageHtml(msg, file, vcf) + '</tr>';
			}
			tmp += '</tbody>';
			
			if (cmp.messages.length == 0)
			{
				// No messages are sorted last in the list
				htmlNoMessages += tmp;
			}
			else if (numWarnings == 0)
			{
				// No warnings are sorted in the middle
				htmlNoWarnings += tmp;
			}
			else
			{
				// Items with at least one warning are sorted first
				html += tmp;
			}
		}
		
		html += htmlNoWarnings;
		html += htmlNoMessages;
		html += '</table>';
		Doc.element('comparisons').innerHTML = html;

		// Add click handler to VCF files
		var clickableItems = document.getElementsByClassName('vcf-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', gt.vcfLinkOnClick);
		}
		
		// Case summary links
		var cs = document.getElementsByClassName('case-summary');
		for (var i = 0; i < cs.length; i++)
		{
			Events.addEventHandler(cs[i], 'click', Reggie.openCaseSummaryPopupOnEvent);
		}

		Wizard.setCurrentStep(2);
		Doc.show('step-2');
		Doc.show('gorestart');
	}

	
	gt.getMessageHtml = function(message, file, vcf)
	{
		var level = message.warningLevel;
		var icon = null;
		if (level == 'INFO')
		{
			icon = 'ok.png';
		}
		else if (level == 'LOW')
		{
			icon = 'warning_small.png';
		}
		else
		{
			icon = 'warning.png';
		}

		var html = '';
		html += '<td class="dottedleft icon-col"><img src="../images/'+icon+'"></td>';
		html += '<td class="text-col" title="'+(message.messageTooltip || '')+'">'+Strings.encodeTags(message.message) + '</td>';
		html += '<td class="text-col">';
		if (message.specimen)
		{
			html += Strings.encodeTags(message.specimen.patientName);
		}
		html += '</td>';
		html += '<td class="text-col">';
		if (message.otherAssay)
		{
			html += gt.tagCommonPartOfName(file.name, message.otherAssay);
			html += '<span class="link case-summary" data-name="'+Strings.encodeTags(message.otherAssay)+'" title="Show case summary"><img src="../images/case_summary.png"></span>';
			if (message.DO_NOT_USE) html += '<img src="../images/donotuse.png" title="DoNotUse-'+Strings.encodeTags(message.DO_NOT_USE + ': ' + message.DO_NOT_USE_COMMENT)+'">';
			if (message.flagged) html += '<img src="../images/flag.png" title="'+Strings.encodeTags(message.otherAssay)+' is already flagged">';
			if (message.currentVerifiedBy) 
			{
				var vicon = message.currentVerifiedBy == 'LYSATE' ? 'gt-verified-lysate.png' : 'gt-verified.png';
				html += '<img src="../images/'+vicon+'" title="'+Strings.encodeTags(message.otherAssay+' is already verified by '+message.currentVerifiedBy)+'">';
			}
		}
		html += '</td>';
		html += '<td class="icon-col">';
		if (message.vcf)
		{
			html += ' <span class="link vcf-link" data-file-id="'+vcf.fileId+'"';
			html += ' data-item-id2="'+message.id+'" data-file-id2="'+message.vcf.fileId+'"';
			html += ' title="Compare genotypes (hold CTRL, ALT or SHIFT to open in a new popup)"><img src="../images/vcf_file.png"></span>';
		}
		html += '</td>';
		return html;
	}
	
	
	gt.tagCommonPartOfName = function(name1, name2)
	{
		var n1 = name1.split('.');
		var n2 = name2.split('.');
		var i = 0;
		
		var common = '';
		var sep = '.';
		while (i < n1.length && i < n2.length && n1[i] == n2[i])
		{
			common += n1[i] + sep;
			i++;
		}
		
		var diff = '';
		sep = '';
		while (i < n2.length)
		{
			diff += sep + n2[i];
			i++;
			sep = '.';
		}
		
		return '<span class="commonpart">'+Strings.encodeTags(common)+'</span><span class="diffpart">'+Strings.encodeTags(diff)+'</span>';
	}
	
	gt.vcfLinkOnClick = function(event)
	{
		var fileId = Data.int(event.currentTarget, 'file-id');
		var itemId = Data.get(event.currentTarget, 'item-id');
		var fileId2 = Data.int(event.currentTarget, 'file-id2');
		var itemId2 = Data.get(event.currentTarget, 'item-id2');
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		
		var url = 'view_genotypes.jsp?ID=' + App.getSessionId();
		url += '&fileId='+fileId;
		url += '&itemId='+itemId;
		url += '&fileId2='+fileId2;
		url += '&itemId2='+itemId2;
		
		Dialogs.openPopup(url, (specialKey ? 'ViewVcf'+itemId+itemId2 : 'ViewVcf'), 900, 600);
	}

	return gt;
}();

Doc.onLoad(GenoType.initPage);

