var Report = function()
{
	var report = {};
	var debug = 0;
	var manuallySelected = [];
	var selectionIsValid = false;
	var rawDataType;
	var pipeline;
	var annotationTypePipeline;
	
	// Page initialization
	report.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectRawBioAssays', report.selectRawBioAssays);
		Events.addEventHandler('reportSoftware', 'change', report.softwareOnChange);
		Events.addEventHandler('rawBioAssays', 'base-selected', report.setRawBioAssayCallback);
		Events.addEventHandler('rawBioAssays', 'change', report.rawBioAssaysOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', report.validateStep1);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', report.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Report.servlet?ID='+App.getSessionId();
		url += '&cmd=GetRawBioAssayWithoutReport';
		url += '&report='+encodeURIComponent(Data.get('page-data', 'report'));
		Wizard.asyncJsonRequest(url, report.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	report.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		Reggie.loadSoftware('REPORT_SOFTWARE', 'reportSoftware', 'REPORT_TEMPLATE,PARAMETER_SET', Data.get('page-data', 'report'));
		
		Doc.element('reportName').innerHTML = Strings.encodeTags(response.reportName);
		Doc.element('configSection').innerHTML = Strings.encodeTags(response.configSection);
		rawDataType = response.rawDataType;
		pipeline = response.pipeline;
		
		var rawBioAssays = response.rawBioAssays;
		if (rawBioAssays.length > 0)
		{
			var firstPlate = null;
			for (var rawNo=0; rawNo < rawBioAssays.length; rawNo++)
			{
				var raw = rawBioAssays[rawNo];
				
				var libPlate = raw.lib.bioWell ? raw.lib.bioWell.bioPlate : null;
				if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;
				var option = report.createListOption(rawNo+1, raw, raw.DO_NOT_USE == null && libPlate != null && libPlate.name==firstPlate)
				frm.rawBioAssays.options[frm.rawBioAssays.length] = option;
			}
			Events.sendChangeEvent('rawBioAssays');
		}
		else
		{
			Wizard.setInputStatus('rawBioAssays', 'invalid', 'No raw bioassays available for processing');
		}

		Doc.show('step-1');
		Doc.show('goregister');
	}
	
	report.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}

	report.selectRawBioAssays = function()
	{
		var frm = document.forms['reggie'];
		if (frm.rawBioAssays.disabled) return;
		// Reset list
		manuallySelected = [];
		var url = '&resetTemporary=1';
		if (rawDataType) url += '&tmpfilter:STRING:rawDataType='+encodeURIComponent(rawDataType);
		if (pipeline != null)
		{
			if (annotationTypePipeline == null)
			{
				annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
			}
			url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'='+encodeURIComponent(pipeline);
		}
		Dialogs.selectItem('RAWBIOASSAY', 'rawBioAssays', 1, url);
	}

	report.setRawBioAssayCallback = function(event)
	{
		var raw = event.detail;
		
		var opt = Reggie.getListOption('rawBioAssays', raw.id);
		if (opt)
		{
			opt.selected = true;
		}
		else
		{
			manuallySelected[manuallySelected.length] = raw.id;
		}
		
		if (raw.remaining == 0 && manuallySelected.length > 0)
		{
			var url = '../Report.servlet?ID='+App.getSessionId();
			url += '&cmd=GetRawBioAssayWithoutReport';
			url += '&items='+manuallySelected.join(',');
			url += '&report='+encodeURIComponent(Data.get('page-data', 'report'));
			Wizard.showLoadingAnimation('Loading bioassays...');
			Wizard.asyncJsonRequest(url, report.manuallySelected);
		}
		else
		{
			Events.sendChangeEvent('rawBioAssays');
		}
	}

	report.manuallySelected = function(response)
	{
		var rawBioAssays = response.rawBioAssays;
		var frm = document.forms['reggie'];	

		if (rawBioAssays != null && rawBioAssays.length > 0)
		{
			var offset = frm.rawBioAssays.length+1;
			for (var rawNo=0; rawNo < rawBioAssays.length; rawNo++)
			{
				var raw = rawBioAssays[rawNo];
				var option = report.createListOption(rawNo+offset, raw, raw.DO_NOT_USE == null);
				frm.rawBioAssays.options[frm.rawBioAssays.length] = option;
			}
			Events.sendChangeEvent('rawBioAssays');
		}
	}
	
	report.createListOption = function(index, rawItem, selected)
	{
		var lib = rawItem.lib;
		var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
		var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
		
		var tooltip = null;
		var name = (index) + ': ';
		if (libPlate) name += libPlate.name + ' - ';
		name += rawItem.name;
		
		if (rawItem.DO_NOT_USE)
		{
			name += ' [DoNotUse]';
			tooltip = 'DoNotUse-'+Strings.encodeTags(rawItem.DO_NOT_USE+': '+rawItem.DO_NOT_USE_COMMENT);
		}
		else if (rawItem.AutoProcess == 'ReProcess')
		{
			name += ' [R]';
		}
		
		var option = new Option(name, rawItem.id, false, selected);
		if (isYellow) option.className = 'yellow';
		if (tooltip) option.title = tooltip;
		option.rawBioAssay = rawItem;
		return option;
	}

	report.rawBioAssaysOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		selectionIsValid = false;
		var numSelected = 0;
		var numDoNotUse = 0;
		var invalidPipeline = null;

		Wizard.setInputStatus('rawBioAssays');
		Wizard.hideGoNextConfirmation();

		for (var rawNo = 0; rawNo < frm.rawBioAssays.length; rawNo++)
		{
			if (frm.rawBioAssays[rawNo].selected) 
			{
				numSelected++;
				
				var raw = frm.rawBioAssays[rawNo].rawBioAssay;
				if (raw.DO_NOT_USE) numDoNotUse++;
				
				if (pipeline && raw.pipeline && raw.pipeline != pipeline)
				{
					invalidPipeline = Strings.encodeTags(raw.name+' ('+raw.pipeline)+') is not intended for the '+pipeline+' pipeline.';
				}
			}
		}
		if (numSelected == 0)
		{
			Wizard.setInputStatus('rawBioAssays', 'invalid', 'Select at least one item');
			return;
		}
		if (invalidPipeline) 
		{
			Wizard.setInputStatus('rawBioAssays', 'invalid', invalidPipeline);
			return;
		}
		selectionIsValid = true;
		
		if (numDoNotUse > 0)
		{
			Wizard.setInputStatus('rawBioAssays', 'warning', numDoNotUse + ' selected items marked as DoNotUse');
			Wizard.showGoNextConfirmation(true, 'Confirm ' + numDoNotUse + ' items marked as DoNotUse');
		}
		else
		{
			Wizard.setInputStatus('rawBioAssays', 'valid');
		}
	}

	report.softwareOnChange = function(event)
	{
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, report.parameterSetInfoLoaded);
		}
	}

	report.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}

	
	report.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		
		var rawBioAssays = [];
		submitInfo.rawBioAssays = rawBioAssays;
		submitInfo.software = parseInt(frm.reportSoftware.value);
		submitInfo.report = Data.get('page-data', 'report');
			
		for (var rawNo = 0; rawNo < frm.rawBioAssays.length; rawNo++)
		{
			if (frm.rawBioAssays[rawNo].selected) 
			{
				var raw = {};
				raw.id = frm.rawBioAssays[rawNo].rawBioAssay.id;
				rawBioAssays[rawBioAssays.length] = raw;
			}
		}

		var url = '../Report.servlet?ID='+App.getSessionId();
		url += '&cmd=CreateReport';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, report.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	report.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	
	return report;
}();

Doc.onLoad(Report.initPage);

