var ImportSpecimen = function()
{
	var specimen = {};
	var debug = 0;
	var importItems;

	var autoAnalyzeIsValid = false;

	specimen.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', specimen.validateStep1);
		Events.addEventHandler('validate-all', 'click', specimen.fullyValidateAll);
		Events.addEventHandler('lessMessages', 'change', specimen.showLessMessages);
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', specimen.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', specimen.validateStep2);
		Events.addEventHandler('clusters', 'change', specimen.clusterOnChange);
		Events.addEventHandler('autoAnalyzeYes', 'click', specimen.autoAnalyzeOnChange);
		Events.addEventHandler('autoAnalyzeNo', 'click', specimen.autoAnalyzeOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('goregister', Wizard.goRegister);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', specimen.submit);
		
		Wizard.showLoadingAnimation('Scanning for JSON files...', 'scan-json-progress');
		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=GetImportInformation';
		Wizard.asyncJsonRequest(url, specimen.initializeStep1);
	}

	// --- Step 1 -----------------------------------
	specimen.initializeStep1 = function(response)
	{
		importItems = response.importItems;
		if (importItems.length == 0)
		{
			Wizard.setFatalError('No items available for importing.');
			return;
		}
		
		importItems.sort(specimen.sortByValidatedAndFileDate);
		
		var numJsonFiles = 0;
		var numNoJsonFiles = 0;
		var numStrayFastq = 0;
		var numValidated = 0;
		var numImportable = 0;
		var numWithWarnings = 0;
		var numNew = 0;
		var numConvertToSpecimen = 0;
		var numFlowCells = 0;
		var lastFlowCellId = null;
		var html = '';
		for (var itemNo = 0; itemNo < importItems.length; itemNo++)
		{
			var item = importItems[itemNo];
			var rowClass = 'highlight';
			if (item.jsonFile)
			{
				numJsonFiles++;
				if (item.jsonFile.fullyValidated) numValidated++;
				if (item.jsonFile.flowCellId != lastFlowCellId)
				{
					numFlowCells++;
					if (lastFlowCellId) rowClass += ' newflowcell';
					lastFlowCellId = item.jsonFile.flowCellId;
				}
			}
			else if (item.itemType == 'StrayFastq')
			{
				numStrayFastq++;
			}
			else
			{
				numNoJsonFiles++;
			}
			rowClass += (numFlowCells % 2 == 0 ? ' evenflowcell': ' oddflowcell')
			html += '<tr class="'+rowClass+'" id="itemRow.'+itemNo+'">';
			html += specimen.createTableRowColumns(item, itemNo);
			html += '</tr>';
			if (item.canImport) 
			{
				numImportable++;
				if (item.jsonFile && item.jsonFile.warnings) numWithWarnings++;
				if (!item.id) numNew++;
				if (item.convertToSpecimen) numConvertToSpecimen++;
			}
		}

		html += '</tbody>';
		html += '</table>';
		Doc.element('itemTable').innerHTML = html;
		
		specimen.addEventHandlers(Doc.element('itemTable'));
		
		Doc.element('num-json-files').innerHTML = numJsonFiles;
		Doc.element('num-no-json-files').innerHTML = numNoJsonFiles;
		Doc.element('num-validated-files').innerHTML = numValidated;
		Doc.element('num-importable').innerHTML = numImportable;
		Doc.element('num-importable-with-warnings').innerHTML = numWithWarnings;
		Doc.element('num-importable-new').innerHTML = numNew;
		Doc.element('num-importable-convert').innerHTML = numConvertToSpecimen;
		Doc.element('num-stray-fastq-files').innerHTML = numStrayFastq;
		if (numValidated < numJsonFiles) Doc.show('validate-all', 'inline');
		
		Doc.show('step-1');
		Doc.show('gonext');
		if (numImportable == 0) 
		{
			Wizard.showGoNextConfirmation(false, 'There are no JSON files that can be imported');
			Doc.addClass('gonext', 'disabled');
		}
		else
		{
			Wizard.hideGoNextConfirmation();
			Doc.removeClass('gonext', 'disabled');
		}
	}
	
	specimen.createTableRowColumns = function(item, itemNo)
	{
		var jsonFile = item.jsonFile;
		item.isStrayFastq = item.itemType == 'StrayFastq';
		item.canImport = item.name && jsonFile && jsonFile.fullyValidated && !jsonFile.errors;

		var html = '';
		var icon = null;
		if (jsonFile)
		{
			icon = 'ok.png';
			if (jsonFile.warnings) icon = 'warning.png';
			if (!jsonFile.fullyValidated) icon = 'info.png';
			if (jsonFile.errors) icon = 'error.png';
		}

		html += '<th>'+(itemNo+1)+'</th>';
		html += '<td class="dottedleft">';
		if (item.isStrayFastq)
		{
			html += item.itemType;
		}
		else if (item.name)
		{
			html += Strings.encodeTags(item.name);
			if (!item.id) html += ' <img src="../images/new.png" title="This item will be created by the importer">';
			if (item.itemType) html += ' <span class="itemsubtype">('+Strings.encodeTags(item.itemType)+')</span>';
		}
		else
		{
			html += '-';
		}
		html += '<td class="dottedleft">';
		if (item.currentCase)
		{
			html += Strings.encodeTags(item.currentCase.name);
			html += '<img src="../images/case_summary.png" class="link case-summary" data-name="'+Strings.encodeTags(item.currentCase.name)+'" title="Show case summary (popup window)">';
		}
		else
		{
			html += '-';
		}
		html += '</td>';
		if (jsonFile)
		{
			html += '<td class="dottedleft" style="white-space: nowrap;">';
			html += Strings.encodeTags(jsonFile.name);
			html += ' <span class="link download-file" data-file-no="'+itemNo+'" title="Download and view this file"><img src="../images/download.png"></span>';
			html += '<br><span class="file-data">('+Numbers.formatBytes(jsonFile.size)+'; '+Reggie.reformatDate(jsonFile.lastModified)+')</span>';
			html += '</td>';
			html += '<td style="white-space: nowrap;">';
			html += Strings.encodeTags(jsonFile.flowCellId);
			html += '<br><span class="file-data">('+Strings.encodeTags(jsonFile.sequencingStartDate)+')</span>';
			html += '</td>';
			html += '<td>';
			html += Reggie.formatCount(jsonFile.PF_READS, '-');
			html += '</td>';
			html += '<td class="fastq" style="white-space: nowrap;">';
			if (jsonFile && jsonFile.fastq && jsonFile.fastq.length)
			{
				for (var fqNo = 0; fqNo < jsonFile.fastq.length; fqNo++)
				{
					var fq = jsonFile.fastq[fqNo];
					html += Strings.encodeTags(fq.name);
					html += ' <span class="file-data">('
					if (fq.size > 0) 
					{
						html += Numbers.formatBytes(fq.size);
					}
					else
					{
						html += 'missing';
					}
					html += ')</span><br>';
				}
			}
			html += '</td>';
		}
		else if (item.isStrayFastq)
		{
			html += '<td class="dottedleft" colspan="4" style="text-align: left;">';
			html += Strings.encodeTags(item.name);
			html += ' <span class="file-data">('
			html += Numbers.formatBytes(item.size);
			html += '; '+ Reggie.reformatDate(item.lastModified);
			html += ')</span>';
			html += '</td>';
		}
		else
		{
			html += '<td class="dottedleft" colspan="4" style="text-align: left;">';
			if (item.externalSpecimenExists)
			{
				html += '<b>Specimen exists:</b> '+Strings.encodeTags(item.externalSpecimenExists);
				if (item.externalOperator) html += '/'+Strings.encodeTags(item.externalOperator);
			}
			if (item.samplingDate)
			{
				html += '; <b>Sampling date:</b> '+Reggie.reformatDate(item.samplingDate);
			}
			if (item.registered)
			{
				html += '; <b>Registration date:</b> '+Reggie.reformatDate(item.registered);
			}
			if (item.otherPathNote)
			{
				html += '; <b>Other path note:</b> '+Strings.encodeTags(item.otherPathNote);
			}
			html += '</td>';
		}
		html += '<td class="icon">';
		if (icon) html+= '<img src="../images/'+icon+'">';
		html += '</td>';
		html += '<td class="valid dottedleft">';
		if (item.canImport)
		{
			html += '<div class="action">';
			html += '<label><input type="checkbox" id="import.'+itemNo+'" class="import-action" data-item-no="'+itemNo+'">';
			var newOrExisting = item.id ? 'existing' : 'new';
			if (item.endPoint.id == 'SPECIMEN' && item.itemType == 'Specimen')
			{
				html += 'Import to '+newOrExisting+' <b>Specimen</b> only</label>';
			}
			else
			{
				html += 'Import to '+newOrExisting+' <b>'+Strings.encodeTags(item.itemType) + '</b> ›› <b>'+item.endPoint.name+'</b></label>';
			}
			if (item.endPoint.id != 'DEMUX' && item.endPoint.ordinal >= item.endPointRNA.ordinal)
			{
				html += '<br><label><input type="checkbox" id="flagrna.'+itemNo+'" disabled>Flag RNA</label>';
			}
			html += '</div>';
		}
		var startNo = 1;
		if (jsonFile)
		{
			if (jsonFile.errors)
			{
				html += '<div class="messagecontainer error"><ol start="'+startNo+'">';
				for (var errNo = 0; errNo < jsonFile.errors.length; errNo++)
				{
					var msg = jsonFile.errors[errNo];
					if (msg.indexOf('[HTML]')==0)
					{
						html += '<li>'+msg.replace('[HTML]', '');
					}
					else
					{
						html += '<li>'+Strings.encodeTags(msg);
					}
				}
				html += '</ol></div>';
				startNo += jsonFile.errors.length;
			}
			else if (!jsonFile.fullyValidated)
			{
				html += '<div class="action">';
				html += 'This JSON file has not been fully validated.';
				html += '<span class="fully-validate link" data-item-no="'+itemNo+'">Validate it!</span>';
				html += '</div>';
			}
			if (jsonFile.fullyValidated && (item.otherPathNote || item.liasonComment))
			{
				html += '<div class="notes">';
				html += Strings.encodeTags(item.otherPathNote);
				html += (item.otherPathNote && item.liasonComment ? '; ' : '')+Strings.encodeTags(item.liasonComment);
				html += '</div>';
			}
			if (jsonFile.warnings && jsonFile.fullyValidated)
			{
				var allVisible = jsonFile.warnings.length < 6 ?' allvisible':'';
				html += '<div class="messagecontainer note minimal '+allVisible+'" id="warnings.'+itemNo+'">';
				html += '<ol start="'+startNo+'">';
				for (var warnNo = 0; warnNo < jsonFile.warnings.length; warnNo++)
				{
					var msg = jsonFile.warnings[warnNo];
					if (msg.indexOf('[HTML]')==0)
					{
						html += '<li>'+msg.replace('[HTML]', '');
					}
					else
					{
						html += '<li>'+Strings.encodeTags(msg);
					}
				}
				html += '</ol>';
				html += '</div>';
				html += '<div class="morewarnings'+allVisible+'">'+jsonFile.warnings.length+' warnings - <span class="showmore-warnings link" data-item-no="'+itemNo+'">Show more...</span></div>';
				startNo += jsonFile.warnings.length;
			}
		}
		html += '</td>';
		return html;
	}
	
	specimen.addEventHandlers = function(topElement)
	{
		Forms.linkCheckboxesWithLabels(topElement);
		var checkboxes = topElement.getElementsByClassName('import-action');
		for (var chkNo = 0; chkNo < checkboxes.length; chkNo++)
		{
			Events.addEventHandler(checkboxes[chkNo], 'click', specimen.toggleImport);
		}
		
		var downloadable = topElement.getElementsByClassName('download-file');
		for (var fileNo = 0; fileNo < downloadable.length; fileNo++)
		{
			Events.addEventHandler(downloadable[fileNo], 'click', specimen.downloadJsonFile);
		}

		var showMoreWarnings = topElement.getElementsByClassName('showmore-warnings');
		for (var itemNo = 0; itemNo < showMoreWarnings.length; itemNo++)
		{
			Events.addEventHandler(showMoreWarnings[itemNo], 'click', specimen.toggleWarnings);
		}
		
		var validateFile = topElement.getElementsByClassName('fully-validate');
		for (var itemNo = 0; itemNo < validateFile.length; itemNo++)
		{
			Events.addEventHandler(validateFile[itemNo], 'click', specimen.fullyValidate);
		}
	
		var cs = topElement.getElementsByClassName('case-summary');
		for (var i = 0; i < cs.length; i++)
		{
			Events.addEventHandler(cs[i], 'click', Reggie.openCaseSummaryPopupOnEvent);
		}
	}
	
	specimen.fullyValidateAll = function(event)
	{
		Doc.element('itemTable').innerHTML = '';
		Doc.hide('validate-all');
		Doc.hide('step-1');
		Doc.hide('gonext');
		
		Wizard.showLoadingAnimation('Validating all...', 'scan-json-progress');
		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=GetImportInformation';
		url += '&validateAll=1';
		Wizard.asyncJsonRequest(url, specimen.initializeStep1);
	}
	
	specimen.fullyValidate = function(event)
	{
		var itemNo = Data.get(event.currentTarget, 'item-no');
		var file = importItems[itemNo].jsonFile;
		
		Wizard.showLoadingAnimation('Validating '+file.name+'...');
		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=GetImportInformation';
		url += '&file='+encodeURIComponent(file.name);
		url += '&itemNo='+itemNo;
		Wizard.asyncJsonRequest(url, specimen.singleFileValidated);
	}
	
	specimen.singleFileValidated = function(response)
	{
		var items = response.importItems;
		if (items.length != 1)
		{
			Wizard.setFatalError('Expected information about a single JSON file: '+items.length);
			return;
		}

		var item = items[0];
		var itemNo = item.itemNo;
		importItems[itemNo] = item;
		
		html = specimen.createTableRowColumns(item, itemNo);
		var rowElement = Doc.element('itemRow.'+itemNo);
		rowElement.innerHTML = html;
		specimen.addEventHandlers(rowElement);

		if (item.canImport)
		{
			Wizard.hideGoNextConfirmation();
			Doc.removeClass('gonext', 'disabled');
		}

	}
	
	
	specimen.toggleWarnings = function(event)
	{
		var itemNo = Data.get(event.currentTarget, 'item-no');
		var warningsDiv = Doc.element('warnings.'+itemNo);
		Doc.addOrRemoveClass(warningsDiv, 'minimal');
		event.currentTarget.innerHTML = warningsDiv.classList.contains('minimal') ? 'Show more...' : 'Show less...';
	}

	specimen.toggleImport = function(event)
	{
		var itemNo = Data.get(event.currentTarget, 'item-no');
		var flag = Doc.element('flagrna.'+itemNo);
		if (flag) flag.disabled = !event.currentTarget.checked;
	}
	
	specimen.showLessMessages = function(event)
	{
		Doc.addOrRemoveClass('importItemsTable', 'less-messages', event.currentTarget.checked);
	}

	specimen.downloadJsonFile = function(event)
	{
		var itemNo = Data.get(event.currentTarget, 'file-no');
		var file = importItems[itemNo].jsonFile;

		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadJsonFile';
		url += '&file='+encodeURIComponent(file.name);
		
		Dialogs.openPopup(url, file.name, 800, 800);
	}

	specimen.validateStep1 = function(event)
	{
		Wizard.hideGoNextConfirmation();
		var frm = document.forms['reggie'];
		for (var itemNo = 0; itemNo < importItems.length; itemNo++)
		{
			var chk = frm['import.'+itemNo];
			if (chk && chk.checked) return;
		}
		Wizard.showGoNextConfirmation(false, 'Please select at least one item!');
		event.preventDefault();
	}

	// --- Step 2 -----------------------------------
	specimen.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadProtocols('MERGE_PROTOCOL', 'mergeProtocol', 'PIPELINE,MERGE_TYPE', 'RNAseq,Import');
		Reggie.loadSoftware('MERGE_SOFTWARE', 'mergeSoftware', 'PIPELINE,MERGE_TYPE,PARAMETER_SET', 'RNAseq,Import');

		// Load clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo&config=demux/execute-import';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, specimen.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	specimen.validateStep2 = function(event)
	{
		if (!autoAnalyzeIsValid) event.preventDefault();
	}
	
	specimen.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		frm.clusters.length = 0;
		var clusters = response.hosts;
		Doc.element('autoAnalyzeNo').disabled = false;
		if (clusters.length == 0)
		{
			Doc.element('autoAnalyzeNo').checked = true;
			autoAnalyzeIsValid = true;
			Wizard.setInputStatus('autoAnalyze', 'valid');
			Wizard.setInputStatus('clusters', 'warning', 'No available clusters');
			return;
		}
		
		Doc.element('autoAnalyzeYes').disabled = false;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
		}
		specimen.autoAnalyzeOnChange();
	}
	
	specimen.autoAnalyzeOnChange = function()
	{
		Wizard.setInputStatus('autoAnalyze');
		autoAnalyzeIsValid = false;
		
		var frm = document.forms['reggie'];
		var yesChecked = Doc.element('autoAnalyzeYes').checked;
		var noChecked = Doc.element('autoAnalyzeNo').checked;
		
		frm.mergeProtocol.disabled = !yesChecked;
		frm.mergeSoftware.disabled = !yesChecked;
		frm.clusters.disabled = !yesChecked;
		frm.priority.disabled = !yesChecked;
		frm.partition.disabled = !yesChecked;
		frm.autoConfirm.disabled = !yesChecked;
		frm.debug.disabled = !yesChecked;

		if (!yesChecked && !noChecked)
		{
			Wizard.setInputStatus('autoAnalyze', 'invalid', 'Must select Yes or No');
			return;
		}
		
		autoAnalyzeIsValid = true;
		Wizard.setInputStatus('autoAnalyze', 'valid');
		
		if (yesChecked) specimen.clusterOnChange();
	}

	
	specimen.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
		if (cluster.partitions && cluster.partitions.length > 0)
		{
			frm.partition.length = 0;
			for (var pNo = 0; pNo < cluster.partitions.length; pNo++)
			{
				var p = cluster.partitions[pNo];
				var title = p.name;
				if (p.description) title += ' ('+p.description+')';
				frm.partition[frm.partition.length] = new Option(title, p.value, p['default'], p['default']);
			}
			Doc.show('job-partition');
		}
		else
		{
			Doc.hide('job-partition');
		}
	}

	/**
		Sort by file date of JSON files. Items without a file are sorted last by registration date and name.
	*/
	specimen.sortByValidatedAndFileDate = function(a, b)
	{
		var fa = a.jsonFile;
		var fb = b.jsonFile;
		
		if (fa && fb)
		{
			if (fa.fullyValidated != fb.fullyValidated)
			{
				return fa.fullyValidated ? -1 : 1;
			}
			if (fa.sequencingStartDate != fb.sequencingStartDate)
			{
				return fa.sequencingStartDate < fb.sequencingStartDate ? -1 : 1;
			}
			if (fa.flowCellId != fb.flowCellId)
			{
				return fa.flowCellId < fb.flowCellId ? -1 : 1;
			}
			if (fa.lastModified != fb.lastModified) 
			{
				return fa.lastModified < fb.lastModified ? -1 : 1;
			}
			return fa.name < fb.name ? -1 : 1;
		}
		else if (fa)
		{
			return -1;
		}
		else if (fb)
		{
			return 1;
		}
		if (a.itemType == 'StrayFastq' && b.itemType == 'StrayFastq')
		{
			if (a.lastModified != b.lastModified)
			{
				return a.lastModified < b.lastModified ? -1 : 1;
			}
		}
		return specimen.sortBySamplingDate(a, b);
	}

	// Sort entries by sampling date or registered date
	specimen.sortBySamplingDate = function(a, b)
	{
		var ra = a.samplingDate || a.registered;
		var rb = b.samplingDate || b.registered;
		if (ra && rb) 
		{
			if (ra != rb) return ra < rb ? -1 : 1;
		}
		else if (ra)
		{
			return -1;
		}
		else if (rb)
		{
			return 1;
		}
		return specimen.sortByName(a, b);
	}

	// Sort entries by name
	specimen.sortByName = function(a, b)
	{
		return a.name == b.name ? 0 : (a.name < b.name ? -1 : 1);
	}

	specimen.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		submitInfo.operator = frm.operator.value;
		submitInfo.getDebugMessages = debug ? true : false;
		
		var items = [];
		submitInfo.items = items;
		for (var itemNo = 0; itemNo < importItems.length; itemNo++)
		{
			var chk = frm['import.'+itemNo];
			var flagRNA = frm['flagrna.'+itemNo];
			if (chk && chk.checked)
			{
				var item = importItems[itemNo]
				var tmp = {};
				tmp.id = item.id;
				tmp.jsonFile = item.jsonFile.name;
				tmp.flagRNA = flagRNA && flagRNA.checked;
				items[items.length] = tmp;
			}
		}

		if (Doc.element('autoAnalyzeYes').checked)
		{
			var autoAnalyze = {};
			autoAnalyze.mergeSoftware = parseInt(frm.mergeSoftware.value);
			autoAnalyze.mergeProtocol = parseInt(frm.mergeProtocol.value);
			autoAnalyze.cluster = frm.clusters.value;
			autoAnalyze.priority = parseInt(frm.priority.value);
			if (frm.partition.selectedIndex >= 0)
			{
				autoAnalyze.partition = frm.partition.value;
			}
			autoAnalyze.debug = frm.debug.checked;
			autoAnalyze.autoConfirm = frm.autoConfirm.checked;
			submitInfo.autoAnalyze = autoAnalyze;
		}

		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=ImportFromJSON';
		
		Wizard.showLoadingAnimation('Importing external specimen...', 'import-json-progress');
		Wizard.asyncJsonRequest(url, specimen.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	specimen.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return specimen;
}();

Doc.onLoad(ImportSpecimen.initPage);