var Methylation = function()
{
	var meth = {};
	var debug = 0;
	
	var JOB_IMAGE = { 'DONE': 'ok.png', 'ERROR': 'error.png'};
	
	var methylationItems;
	
	// Page initialization
	meth.initPage = function()
	{
		
		// Step 1

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', meth.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Methylation.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnconfirmedBetaAnalysis';
		Wizard.asyncJsonRequest(url, meth.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	meth.initializeStep1 = function(response)
	{
		methylationItems = response.methylationItems;
		if (methylationItems.length == 0)
		{
			Wizard.setFatalError('No methylation items available for processing.');
			return;
		}
		
		var html = '<table id="methylationTable">';
		html += '<thead class="bg-filled-100">';
		// Header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft" colspan="2">BeadChip</th>';
		html += '<th class="dottedleft" colspan="3">Median int.</th>';
		html += '<th class="dottedleft" colspan="4">Probes</th>';
		html += '<th class="dottedleft" colspan="3">Peaks (Infinium I/II)</th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft" colspan="2">Actions</th>';
		html += '<th class="dottedleft">Comment</th>';
		html += '</tr>';
		
		html += '<tr>';
		html += '<th>Methylation</th>';
		html += '<th class="dottedleft">Sentrix ID</th>';
		html += '<th>Position</th>';
		html += '<th class="dottedleft">Meth</th>';
		html += '<th>Unmeth</th>';
		html += '<th class="icon-col"></th>';
		html += '<th class="dottedleft">Total</th>';
		html += '<th title="Probes with detectionP-value &gt; 0.01.">Bad¹</th>';
		html += '<th>Rate</th>';
		html += '<th class="icon-col"></th>';
		html += '<th class="dottedleft">Meth</th>';
		html += '<th>Unmeth</th>';
		html += '<th class="icon-col"></th>';
		html += '<th class="dottedleft">Warnings</th>';
		html += '<th class="dottedleft">Job</th>';
		html += '<th class="dottedleft">Files</th>';
		html += '<th class="dottedleft"><span id="reanalyze" data-prefix="reanalyze" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Re-analyze</span></th>';
		html += '<th><span id="confirm" data-prefix="confirm" class="interactable link" title="Toggle selection – use CTRL, ALT or SHIFT to clear">Confirm</span></th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		for (var methNo = 0; methNo < methylationItems.length; methNo++)
		{
			var m = methylationItems[methNo];
			var dna = m.dna;
			var chip = m.beadChip;
			var job = m.job;
			var isError = job.status == 'ERROR';
			
			html += '<tr class="highlight ' + (methNo % 4 < 2 ? 'evenrow' : 'oddrow') + '">';
			html += '<td class="prompt">'+m.name+'</td>';
			
			html += '<td class="dottedleft">'+(chip ? Strings.encodeTags(chip.BeadChipId) : '-')+'</td>';
			html += '<td>'+(dna ? Strings.encodeTags(dna.BeadChipPosition) : '-')+'</td>';


			html += '<td class="dottedleft">'+Reggie.formatNumber(m.MedianMeth, null, 2)+'</td>';
			html += '<td>'+Reggie.formatNumber(m.MedianUnmeth, null, 2)+'</td>';
			html += '<td class="icon-col">';
			if (m.MedianMeth != null && m.MedianUnmeth != null && (m.MedianMeth+m.MedianUnmeth<21))
			{
				html += '<img src="../images/warning_small.png" title="Mean signal&lt;10.5">';
			}
			html += '</td>';

			html += '<td class="dottedleft">'+Reggie.formatNumber(m.ProbesRaw, null, 2)+'</td>';
			html += '<td>'+Reggie.formatNumber(m.ProbesBad, null, 2)+'</td>';
			html += '<td>'+(m.ProbesRate == null ? '':Reggie.formatNumber(m.ProbesRate*100, '%', 2))+'</td>';
			html += '<td class="icon-col">';
			if (m.ProbesRate != null && m.ProbesRate<0.99)
			{
				html += '<img src="../images/warning_small.png" title="&gt;1% probes with p&gt;0.01">';
			}
			html += '</td>';
			html += '<td class="dottedleft">';
			var peakWarning = null;
			if (m.PeakMethI != null || m.PeakMethII != null)
			{
				if ((m.PeakMethI != null && m.PeakMethI < 0.8) || (m.peakMethII && m.PeakMethII < 0.8))
				{
					peakWarning = 'Peak Meth &lt;0.8';
				} 
				html += Reggie.formatNumber(m.PeakMethI, null, 1)+'/'+Reggie.formatNumber(m.PeakMethII, null, 1);
			}
			html += '</td>';
			html += '<td>';
			if (m.PeakUnmethI != null || m.PeakUnmethII != null)
			{
				if ((m.PeakUnmethI != null && m.PeakUnmethI > 0.2) || (m.PeakUnmethII != null && m.PeakUnmethII > 0.2))
				{
					peakWarning = 'Peak Unmeth &gt;0.2';
				}
				html += Reggie.formatNumber(m.PeakUnmethI, null, 1)+'/'+Reggie.formatNumber(m.PeakUnmethII, null, 1);
			}
			html += '</td>';
			html += '<td class="icon-col">';
			if (m.adjustmentPlots)
			{
				html += '<span class="link pdf-link" data-file-id="'+m.adjustmentPlots.id+'" data-item-id="'+m.id+'" title="View adjustment plots'+(peakWarning?': '+peakWarning:'')+'">';
				html += '<img src="../images/'+(peakWarning?'pdffile_warning.png':'pdffile.png')+'">';
				html += '</span>';
			}
			html += '</td>';
			html += '<td class="dottedleft">'+Strings.encodeTags(m.AnalysisFlag)+'</td>';

			html += '<td class="dottedleft">';
			html += '<span class="link item-link" data-item-type="JOB" data-item-id="'+job.id+'" title="'+Strings.encodeTags(job.statusMessage)+'"><img src="../images/'+JOB_IMAGE[job.status]+'"></span>';
			html += '</td>';
			if (m.DataFilesFolder && job.server)
			{
				html += '<td class="dottedleft">';
				html += '<span class="link file-link" data-server="'+Strings.encodeTags(job.server)+'" data-folder="'+Strings.encodeTags(m.DataFilesFolder)+'" data-item-id="'+m.id+'" title="View generated files"><img src="../images/remote_folder.png"></span>';
				html += '</td>';
			}
			else
			{
				html += '<td class="dottedleft">-</td>';
			}
			
			var reanalyzeChecked = isError ? ' checked' : '';
			var confirmChecked = !isError ? ' checked' : '';
			var confirmDisabled = '';
			html += '<td class="dottedleft"><input type="checkbox" name="reanalyze.'+m.id+'"'+reanalyzeChecked+'></td>';
			html += '<td><input type="checkbox" name="confirm.'+m.id+'"'+confirmChecked+confirmDisabled+'></td>';
			html += '<td class="dottedleft comment"><input type="text" name="comment.'+m.id+'" value="'+Strings.encodeTags(m.comment)+'"></td>';
			html += '</tr>';

		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('methylationItems').innerHTML = html;
		
		Events.addEventHandler('reanalyze', 'click', meth.toggleSelection);
		Events.addEventHandler('confirm', 'click', meth.toggleSelection);
		
		// Add click handler to items
		var clickableItems = document.getElementsByClassName('item-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Items.itemOnClick);
		}
		
		// Add click handler to file links
		var clickableItems = document.getElementsByClassName('file-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', meth.fileLinkOnClick);
		}

		// Add click handler to PDF files
		var clickableItems = document.getElementsByClassName('pdf-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Files.viewFileOnClick);
		}

		Doc.show('step-1');
		Doc.show('goregister');
	}
	
	meth.fileLinkOnClick = function(event)
	{
		var folder = Data.get(event.currentTarget, 'folder');
		var server = Data.get(event.currentTarget, 'server');
		var itemId = Data.get(event.currentTarget, 'item-id');
		server = server.replace(/\s*\[.*\]/, '');
		var url = '../analysis/view_remote_files.jsp?ID=' + App.getSessionId();
		url += '&clusterId='+server;
		url += '&path='+encodeURIComponent(folder);
		url += '&itemType=DERIVEDBIOASSAY&itemId='+itemId;
		Dialogs.openPopup(url, 'ViewProjectFiles', 750, 500);
	}
	
	meth.toggleSelection = function(event)
	{
		var prefix = Data.get(event.currentTarget, 'prefix');
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		
		var frm = document.forms['reggie'];
		for (var methNo = 0; methNo < methylationItems.length; methNo++)
		{
			var m = methylationItems[methNo];
			var chk = frm[prefix+'.'+m.id];
			if (chk && !chk.disabled)
			{
				chk.checked = specialKey ? false : !chk.checked;
			}
		}
	}
	
	meth.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		
		var all = [];
		submitInfo.methylationItems = all;
		for (var methNo = 0; methNo < methylationItems.length; methNo++)
		{
			var m = methylationItems[methNo];
			var tmp = {};
			tmp.id = m.id;
			
			tmp.reanalyze = frm['reanalyze.'+m.id].checked;
			tmp.confirm = frm['confirm.'+m.id].checked;
			tmp.comment = frm['comment.'+m.id].value;
			
			all[all.length] = tmp;
		}
		
		var url = '../Methylation.servlet?ID='+App.getSessionId();
		url += '&cmd=ConfirmBetaAnalysis';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, meth.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	meth.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	return meth;
}();

Doc.onLoad(Methylation.initPage);

