var MipsPlate = function()
{
	var mips = {};
	var barcodesByName = [];
	var hasInitProtocols = false;
	
	var debug = 0;
	
	var plateTypeNormalizedDNA;
	var subtypeDna;
	var subtypeDnaNormalized;

	// Page initialization
	mips.initPage = function()
	{
		
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', mips.validateStep1);

		Buttons.addClickHandler('btnAddAliquotPlates', mips.addAliquotPlates);
		Buttons.addClickHandler('btnRemoveAliquotPlates', mips.removeAliquotPlates);
		Buttons.addClickHandler('moveUp', mips.movePlate);
		Buttons.addClickHandler('moveDown', mips.movePlate);
		Events.addEventHandler('plates', 'base-selected', mips.aliquotPlateSelected);

		Events.addEventHandler('pool-row', 'click', mips.togglePool);
		Events.addEventHandler('pool-row', 'mouseover', mips.highlightPool);
		Events.addEventHandler('pool-row', 'mouseout', mips.highlightPool);
		Events.addEventHandler('plate-row', 'click', mips.togglePool);
		Events.addEventHandler('plate-row', 'mouseover', mips.highlightGroup);
		Events.addEventHandler('plate-row', 'mouseout', mips.highlightGroup);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', mips.submit);
		
		var url = '../Mips.servlet?ID='+App.getSessionId();
		url += '&cmd=GetNextAutoGeneratedPlateName';
		url += '&bioPlateType=MIPS';
		Wizard.showLoadingAnimation('Loading next MIPs plate');
		Wizard.asyncJsonRequest(url, mips.nextPlateNameLoaded);
	}
	
	mips.nextPlateNameLoaded = function(response)
	{
		Plate.name = response.name;
		Doc.element('plateName').innerHTML = Strings.encodeTags(Plate.name);

		var columns = 12;
		var rows = 8;
		Plate.init(rows, columns, WellPainter);
		Plate.paint(Plate.getWells());
		
		Doc.show('step-1');
		Doc.show('gocancel');
		Doc.show('goregister');
		
		window.addEventListener('beforeunload', mips.confirmLeaveWizard);
		Wizard.keepSessionAlive();
	}
	
	mips.toggleInfo = function(event)
	{
		var show = event.currentTarget.checked;
		Doc.addOrRemoveClass('plate', 'hide-'+event.currentTarget.id, !show);
	}
	

	
	/**
		Ask for confirmation to abort if DNA has been placed on the plate.
	*/
	mips.confirmLeaveWizard = function(event)
	{
		if (Plate.getUsed().length > 0)
		{
			event.preventDefault();
		}
	}

	mips.validateStep1 = function(event)
	{
		var numErrors = 0;
		var numWarnings = 0;
		var numDna = 0;
		var wells = Plate.getWells();
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			var dna = well.extract;
			if (well.hasWarning())
			{
				numWarnings++;
			}
			if (well.hasError())
			{
				numErrors++;
			}
			else
			{
				if (dna && dna.id) numDna++;
			}
		}

		if (numErrors > 0)
		{
			event.preventDefault();
			return;
		}
		
		if (numDna == 0)
		{
			Wizard.showGoNextConfirmation(false, 'The plate is empty!');
			event.preventDefault();
			return;
		}
	}

	mips.movePlate = function(event)
	{
		var moveDown = Data.int(event.currentTarget, 'down');
		Forms.moveListOptions('plates', moveDown);
		mips.layoutFromAliquotPlates();
	}
	
	
	// Temporary array for holding selected aliqout plates before loading more information via the servlet
	var newPlates;
	/**
		Open dialog for adding more aliquot plates.
	*/
	mips.addAliquotPlates = function(event)
	{
		var frm = document.forms['reggie'];
		var emptyColumn = Plate.getFirstEmptyColumn();
		if (emptyColumn == -1)
		{
			Forms.showNotification(event.currentTarget, 'There are no more empty columns.');
			return;
		}

		newPlates = []; // Clear the temp array
		var current = []; // Filter out currently included plates
		var plates = frm.plates;
		for (var plateNo = 0; plateNo < plates.length; plateNo++)
		{
			current[current.length] = plates[plateNo].value;
		}
		
		var url = 'select_aliquot_plates.jsp?ID='+App.getSessionId();
		if (current.length > 0) url += '&ignore='+current.join(',');
		Dialogs.openPopup(url, 'SelectAliquotPlates', 900, 600);
	}
	
	/**
		Removes the selected aliquot plates.
	*/
	mips.removeAliquotPlates = function()
	{
		var frm = document.forms['reggie'];
		var plates = frm.plates;
		for (var plateNo = 0; plateNo < plates.length; plateNo++)
		{
			if (plates[plateNo].selected) plates[plateNo--] = null;
		}
		mips.layoutFromAliquotPlates();
	}

	
	/**
		Callback when a new aliquot plate has been selected for including on the MIPs plate.
		Adds selected plates to the temp array. When all have been collected, a request
		to server is made to load more information about the DNA on the plate(s).
	*/
	mips.aliquotPlateSelected = function(event)
	{
		var frm = document.forms['reggie'];
		var name = event.detail.name;
		var id = event.detail.id;
		
		// Check if the selected plate is a new plate or not
		var plateList = frm.plates;
		var isNew = true;
		for (var i = 0; i < plateList.length; i++)
		{
			if (plateList[i].plate.id == id)
			{
				isNew = false;
				break;
			}
		}
		if (isNew)
		{
			newPlates[newPlates.length] = id;
		}
		
		// Load information about DNA on the plates
		if (event.detail.remaining == 0 && newPlates.length > 0)
		{
			var url = '../Mips.servlet?ID='+App.getSessionId();
			url += '&cmd=GetUnusedAliquotsFromPlates';
			url += '&plates='+newPlates.join(',');
			Wizard.showLoadingAnimation('Loading aliquots from ' + encodeURIComponent(event.detail.name));
			Wizard.asyncJsonRequest(url, mips.aliquotsAndPlatesLoaded);
			newPlates = [];
		}
	}
	
	/**
		Callback after we have got DNA information for the new plates.
	*/
	mips.aliquotsAndPlatesLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var plates = response.plates;
		var emptyPlates = [];
		for (var plateNo = 0; plateNo < plates.length; plateNo++)
		{
			var plate = plates[plateNo];
			var aliquots = plate.aliquots;
			if (aliquots.length > 0)
			{
				var option = new Option(plate.name + ' ('+aliquots.length+' aliquots)', plate.id);
				option.plate = plate;
				frm.plates[frm.plates.length] = option;
			}
			else
			{
				emptyPlates[emptyPlates.length] = plate.name;
			}
		}
		if (emptyPlates.length > 0)
		{
			var isOrAre = emptyPlates.length == 1 ? ' is' : ' are';
			Forms.showNotification('btnAddAliquotPlates', Strings.encodeTags(emptyPlates.join(', ')) + isOrAre + '  empty');
		}
		mips.layoutFromAliquotPlates();
	}
	
	/**
		Auto-layout the MIPs plate from the selected aliquot plates.
	*/
	mips.layoutFromAliquotPlates = function()
	{
		var frm = document.forms['reggie'];
		var wells = Plate.getWells();
		Plate.clear(wells, true);
		Doc.hide('pool-row');
		Doc.hide('plate-row');
		Doc.hide('unplaced-aliquots');
		Wizard.hideGoNextConfirmation();
		COLUMN_DATA = new Array(Plate.columns);

		var plates = frm.plates;
		var mipsCurrentColumn = -1; // Current column on MIPs plate
		PLATE_LOOP:
		for (var plateNo = 0; plateNo < plates.length; plateNo++)
		{
			var plate = plates[plateNo].plate;
			var selected = plates[plateNo].selected;
			var aliquots = plate.aliquots;
			var aliquotCurrentColumn = -1; // Current column on the aliquot plate
			
			plate.numPlacedAliquots = 0;
			plate.columnSpan = 0;
			for (var aqNo = 0; aqNo < aliquots.length; aqNo++)
			{
				var aliquot = aliquots[aqNo];
				if (aliquot.bioWell.column != aliquotCurrentColumn)
				{
					mipsCurrentColumn++;
					if (mipsCurrentColumn >= Plate.columns)
					{
						var remain = aliquots.length - aqNo;
						Doc.element('unplaced-aliquots').innerHTML = remain + ' aliquots from ' +Strings.encodeTags(plate.name) + ' could not be placed';
						Doc.show('unplaced-aliquots', 'table-cell');
						plateNo++;
						break PLATE_LOOP;
					}
					aliquotCurrentColumn = aliquot.bioWell.column;
					plate.columnSpan++;
					var col = {};
					col.plateId = plate.id;
					COLUMN_DATA[mipsCurrentColumn] = col;
				}
				var well = Plate.getWell(aliquot.bioWell.row, mipsCurrentColumn);
				well.setExtract(aliquot);
				//well.selected = selected;
				plate.numPlacedAliquots++;
			}
		}
		//if (plateNo < plates.length) frm.plates.length = plateNo;
		
		// Analyze columns on the MIPs plate with resepect to MIPS_Panel and PoolVolume
		for (var colNo = 0; colNo < Plate.columns; colNo++)
		{
			col = COLUMN_DATA[colNo];
			if (!col) 
			{
				col = {};
				COLUMN_DATA[colNo] = col;
			}
			col.numAliquots = 0;
			col.panelCounter = new ItemCounter();
			col.poolVolumeCounter = new ItemCounter();
			for (var rowNo = 0; rowNo < Plate.rows; rowNo++)
			{
				var well = Plate.getWell(rowNo, colNo);
				if (well.extract)
				{
					var aliquot = well.extract;
					col.numAliquots++;
					col.panelCounter.count(aliquot.MIPS_Panel);
					col.poolVolumeCounter.count(aliquot.MIPS_PoolVolume);
				}
			}
			col.MIPS_Panel = col.panelCounter.maxKey;
			col.isMissingMIPSPanel = col.panelCounter.numCounted != col.numAliquots;
			col.hasMultipleMIPSPanels = col.panelCounter.numKeys() > 1;
			col.MIPS_PoolVolume = col.poolVolumeCounter.maxKey;
			col.hasMultiplePoolVolumes = col.poolVolumeCounter.numKeys() > 1;
		}
		
		if (plates.length > 0)
		{
			mips.buildPoolRow();
			mips.buildPlateRow();
		}
		Plate.paint(wells);
		
		var wells = Plate.getWells();
		var numWarnings = 0;
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			if (well.hasWarning())
			{
				numWarnings++;
			}
		}

		if (numWarnings > 0)
		{
			Wizard.showGoNextConfirmation(true, 'Check to verify registration of plate with warnings.');
		}
	}
	
	mips.buildPoolRow = function()
	{
		var html = '<th></th>';
		var pools = [];	// A 'pool' is a number of columns with the same MIPS_Panel
		var currentPool = {};

		for (var colNo = 0; colNo < COLUMN_DATA.length; colNo++)
		{
			var col = COLUMN_DATA[colNo];
			if ((col.MIPS_Panel != currentPool.name) || (col.numAliquots == 0 && currentPool.numAliquots != 0))
			{
				// The column has a different MIPS_Panel than the previous one so we create a new pool
				currentPool = {};
				currentPool.name = col.MIPS_Panel;
				currentPool.startCol = colNo;
				currentPool.columnSpan = 1;
				currentPool.hasMultipleMIPSPanels = col.hasMultipleMIPSPanels;
				currentPool.hasMultiplePoolVolumes = col.hasMultiplePoolVolumes;
				currentPool.isMissingMIPSPanel = col.isMissingMIPSPanel;
				currentPool.numAliquots = col.numAliquots;
				pools[pools.length] = currentPool;
			}
			else if (currentPool)
			{
				// The column has the same MIPS_Panel as the previous one, we make the pool span across multiple columns
				currentPool.columnSpan++;
				currentPool.numAliquots += col.numAliquots;
				if (col.isMissingMIPSPanel) currentPool.isMissingMIPSPanel = true;
				if (col.hasMultipleMIPSPanels) currentPool.hasMultipleMIPSPanels = true;
				if (col.hasMultiplePoolVolumes) currentPool.hasMultiplePoolVolumes = true;
			}
		}
		
		colNo = 0;
		for (var poolNo = 0; poolNo < pools.length; poolNo++)
		{
			var pool = pools[poolNo];
			var poolClass = 'pool link';
			var poolText = null;
						
			COLUMN_DATA[colNo].poolClass = 'pool-left';
			if (pool.columnSpan == 1)
			{
				if (pool.name) COLUMN_DATA[colNo].poolClass += ' pool-right';
			}
			else
			{
				if (pool.name)
				{
					for (var i = 1; i < pool.columnSpan-1; i++)
					{
						COLUMN_DATA[colNo+i].poolClass = 'pool-middle';
					}
					COLUMN_DATA[colNo+pool.columnSpan-1].poolClass = 'pool-right';
				}
			}

			if (pool.numAliquots > 0)
			{
				if (pool.hasMultiplePoolVolumes || pool.isMissingMIPSPanel) 
				{
					poolClass += ' pool-warning';
					if (pool.isMissingMIPSPanel)
					{
						poolText = 'Some aliquots have no MIPS_Panel!';
					}
					else if (pool.hasMultiplePoolVolumes)
					{
						poolText = 'Some aliquots have a different pool volume!';
					}
				}
				if (pool.name) poolClass += ' ' + MipsColor.getColor(pool.name);
				html += '<th id="pool.' + poolNo + '" colspan="'+pool.columnSpan+'"';
				html += ' class="'+poolClass+'"';
				html += ' data-start-col="'+pool.startCol+'"';
				html += ' data-end-col="'+(pool.startCol+pool.columnSpan-1)+'"';
				if (poolText) html += ' title="'+poolText+'"';
				html += '>'+Strings.encodeTags(pool.name || 'No MIPS_Panel')+'</th>';
			}
			else
			{
				poolClass = '';
				if (colNo > 0) poolClass += ' nopool-after-pool';
				if (colNo + pool.columnSpan < 12) poolClass += ' nopool-before-pool';
				html += '<th colspan="'+pool.columnSpan+'"';
				html += ' class="'+poolClass+'"';
				html += '></th>';
			}
			colNo += pool.columnSpan;
		}
		html += '<td style="padding:0;"></td>';
		
		var row = Doc.element('pool-row');
		row.innerHTML = html;
		Doc.show(row);
	}
	
	mips.buildPlateRow = function()
	{
		var html = '<th></th>';
		var totalSpan = 0;
		var colNo = 0;
		for (var plateNo = 0; plateNo < plates.length && colNo < Plate.columns; plateNo++)
		{
			var plate = plates[plateNo].plate;
			var poolClass = 'group link';
			var poolText = null;
			
			if (plate.columnSpan == 1)
			{
				COLUMN_DATA[colNo].groupClass = 'group-left group-right';
			}
			else
			{
				COLUMN_DATA[colNo].groupClass = 'group-left';
				for (var i = 1; i < plate.columnSpan-1; i++)
				{
					COLUMN_DATA[colNo+i].groupClass = 'group-middle';
				}
				COLUMN_DATA[colNo+plate.columnSpan-1].groupClass = 'group-right';
			}
			
			if (plate.numPlacedAliquots != plate.aliquots.length)
			{
				poolClass += ' pool-warning';
				poolText = 'This plate has ' + (plate.aliquots.length - plate.numPlacedAliquots) + ' more aliquots';
			}
			
			html += '<th id="group.' + plateNo + '" colspan="'+plate.columnSpan+'"';
			html += ' class="'+poolClass+'"';
			html += ' data-start-col="' + colNo + '"';
			html += ' data-end-col="' + (colNo+plate.columnSpan-1) + '"';
			if (poolText) html += ' title="'+poolText+'"';
			html += '>'+Strings.encodeTags(plate.name)+'</th>';
			colNo += plate.columnSpan;
		}
		if (colNo < Plate.columns)
		{
			html += '<th class="nopool-after-pool" colspan="'+(Plate.columns-colNo)+'"></th>';
		}
		html += '<td style="padding:0;"></td>';
		var row = document.getElementById('plate-row');
		row.innerHTML = html;
		Doc.show(row);
	}
	
	mips.submit = function()
	{
		var frm = document.forms['reggie'];

		var submitInfo = {};
		
		var mipsPlate = {};
		mipsPlate.name = Plate.name;
		mipsPlate.comments = frm.comments.value;
		mipsPlate.wells = [];
		submitInfo.mipsPlate = mipsPlate;
		
		var aliquotPlates = [];
		submitInfo.aliquotPlates = aliquotPlates
		for (var i = 0; i < frm.plates.length; i++)
		{
			var plate = frm.plates[i].plate;
			var tmp = {};
			tmp.id = plate.id;
			tmp.numAliquots = plate.aliquots.length;
			tmp.numPlacedAliquots = plate.numPlacedAliquots;
			aliquotPlates[i] = tmp;
		}

		var wells = Plate.getWells();
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			var aliquot = well.extract;
			if (aliquot && aliquot.id)
			{
				var tmp = {};
				tmp.row = well.row;
				tmp.column = well.column;
				tmp.aliquot = {};
				tmp.aliquot.id = aliquot.id;
				mipsPlate.wells[mipsPlate.wells.length] = tmp;
			}
		}
		
		var url = '../Mips.servlet?ID='+App.getSessionId();
		url += '&cmd=CreateMIPsPlate';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, mips.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	mips.submissionResults = function(response)
	{
		window.removeEventListener('beforeunload', mips.confirmLeaveWizard);
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	mips.initElements = function(element, autoInit)
	{
		if (autoInit == 'plate-col')
		{
			Events.addEventHandler(element, 'click', mips.toggleColumn);
			Events.addEventHandler(element, 'mouseover', mips.highlightColumn);
			Events.addEventHandler(element, 'mouseout', mips.highlightColumn);
		}
		else if (autoInit == 'plate-row')
		{
			Events.addEventHandler(element, 'click', mips.toggleRow);
			Events.addEventHandler(element, 'mouseover', mips.highlightRow);
			Events.addEventHandler(element, 'mouseout', mips.highlightRow);
		}
		else if (autoInit == 'plate-well')
		{
			Events.addEventHandler(element, 'click', mips.toggleWell);
		}
	}

	// Toggle the selected status of a single well
	mips.toggleWell = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var column = Data.int(event.currentTarget, 'col');
		var well = Plate.getWell(row, column);
		Plate.toggleSelected([well]);
	}

	// Toggle the selected status of a complete row
	mips.toggleRow = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		Plate.toggleSelected(Plate.getRow(row));
	}

	// Toggle the selected status of a complete column
	mips.toggleColumn = function(event)
	{
		var column = Data.int(event.currentTarget, 'col');
		Plate.toggleSelected(Plate.getColumn(column));
	}
	
	// Toggle the selected status of a pool
	mips.togglePool = function(event)
	{
		var startCol = Data.int(event.target, 'start-col');
		var endCol = Data.int(event.target, 'end-col');
		Plate.toggleSelected(Plate.getPool(startCol, endCol));
	}
	
	// Highlight enable/disable all wells in a column
	mips.highlightColumn = function(event)
	{
		var column = Data.int(event.currentTarget, 'col');
		var on = event.type == 'mouseover';
		
		Doc.addOrRemoveClass(event.currentTarget, 'highlight-column', on);
		var wells = Plate.getColumn(column);
		Plate.setHighlight(wells, 'highlight-column', on);
	}

	// Highlight enable/disable all wells in a row
	mips.highlightRow = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var on = event.type == 'mouseover';
		
		Doc.addOrRemoveClass(event.currentTarget, 'highlight-row', on);
		var wells = Plate.getRow(row);
		Plate.setHighlight(wells, 'highlight-row', on);
	}
	
	// Highligt enable/disable all wells in a pool
	mips.highlightPool = function(event)
	{
		var startCol = Data.int(event.target, 'start-col');
		var endCol = Data.int(event.target, 'end-col');
		var on = event.type == 'mouseover';
		Doc.addOrRemoveClass(event.target, 'highlight-pool', on);
		var wells = Plate.getPool(startCol, endCol);
		Plate.setHighlight(wells, 'highlight-pool', on);
	}

	// Highligt enable/disable all wells in a pool
	mips.highlightGroup = function(event)
	{
		var startCol = Data.int(event.target, 'start-col');
		var endCol = Data.int(event.target, 'end-col');
		var on = event.type == 'mouseover';
		Doc.addOrRemoveClass(event.target, 'highlight-group', on);
		var wells = Plate.getPool(startCol, endCol);
		Plate.setHighlight(wells, 'highlight-group', on);
	}

	return mips;
}();

Doc.onLoad(MipsPlate.initPage);
Doc.addElementInitializer(MipsPlate.initElements);

var COLUMN_DATA = [];

var WellPainter = function()
{
	var painter = {};
	
	// Add class indicators for replicates and QC assigned wells
	painter.getClassNameForWell = function(well)
	{
		var cls = '';
		if (well.extract)
		{
			cls += ' primary';
			if (well.extract.MIPS_Panel) cls += ' ' + MipsColor.getColor(well.extract.MIPS_Panel);
		}
		if (well.column == Plate.columns-1) cls += ' last-child';
		if (COLUMN_DATA[well.column]) 
		{
			cls += ' ' + (COLUMN_DATA[well.column].poolClass || '');
			cls += ' ' + (COLUMN_DATA[well.column].groupClass || '');
		}
		return cls;
	}
	
	painter.getWellText = function(well)
	{
		var text = '';
		if (well.extract)
		{
			// The well contains DNA
			var aliquot = well.extract;
			text += '<div class="name">'+Reggie.softLineBreaks(Strings.encodeTags(aliquot.name))+'</div>';
			
			var warningMsg = [];
			if (aliquot.bioWell)
			{
				var dnaWell = aliquot.bioWell;
				text += '<div class="location">['+dnaWell.location+']</div>';
			}
			else
			{
				warningMsg[warningMsg.length] = 'No location!';
			}
	
			if (aliquot.MIPS_PoolVolume != null)
			{
				text += '<div class="volume">'+Numbers.formatNumber(aliquot.MIPS_PoolVolume, 0, 'µl')+'</div>';
				if (aliquot.MIPS_PoolVolume != COLUMN_DATA[well.column].MIPS_PoolVolume)
				{
					warningMsg[warningMsg.length] = 'PoolVolume!';
				}
			}
			if (aliquot.MIPS_Panel != null)
			{
				text += '<div class="mips-panel">'+Strings.encodeTags(aliquot.MIPS_Panel)+'</div>';
			}
			else
			{
				well.setError('No MIPS_Panel');
			}
			
			if (aliquot.MIPS_Form != null || aliquot.MIPS_NormalTumor != null)
			{
				text += '<div class="mips-form">';
				if (aliquot.MIPS_NormalTumor != null) text += '<span>'+Strings.encodeTags(aliquot.MIPS_NormalTumor)+'</span>';
				if (aliquot.MIPS_Form != null) text += '<span>'+Strings.encodeTags(aliquot.MIPS_Form)+'</span>';
				text += '</div>';
			}

			if (aliquot.comment)
			{
				text += '<div class="comment">'+Strings.encodeTags(aliquot.comment)+'</div>';
			}

			if (aliquot.DO_NOT_USE)
			{
				warningMsg[warningMsg.length] = 'DoNotUse-'+Strings.encodeTags(aliquot.DO_NOT_USE);
			}
			if (warningMsg.length > 0)
			{
				well.setWarning(warningMsg.join('; '));
			}
		}
		else
		{
			text = 'empty';
		}

		return text;
	}
	
	return painter;
}();
