var MapData = function()
{
	var map = {};
	var debug = 0;
	
	map.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', map.validateStep1);
		Wizard.initFileSelectionField('file');
		Events.addEventHandler('file', 'change', map.fileChanged);

		Events.addEventHandler('mapFrom', 'change', map.mapFromChanged);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('goexport', Wizard.goRegister);
		Buttons.addClickHandler('godownload', map.download);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', map.submit);
		
		Doc.show('step-1');
		Doc.show('goexport');
		Events.sendChangeEvent('mapFrom');
	}
	
	map.mapFromChanged = function()
	{
		var frm = document.forms['reggie'];
		var fromPnr = frm.mapFrom.value == 'PersonalNo' || frm.mapFrom.value == 'PAD';
		
		Doc.showHide('exportPersonalNo', !fromPnr);
		Doc.showHide('alsoExportPad', !fromPnr);
		Doc.showHide('exportPatientId', fromPnr);
		Doc.showHide('alsoExportCaseId', fromPnr);
		Doc.showHide('alsoExportSpecimenId', fromPnr);
	}
	
	map.fileChanged = function()
	{
		var frm = document.forms['reggie'];
		var file = frm.file.files[0];
		var reader = new FileReader();
		reader.onload = map.fileLoaded;
		reader.readAsText(file.slice(0, 100000)); // Read at most 100kB -- I think we should get at least 2 lines.... 
	}

	map.fileLoaded = function(event)
	{
		var frm = document.forms['reggie'];
		var data = event.target.result;
		var lines = data.split(/[\n\r]+/);

		var lineNo = 0;
		var mapFrom = null;
		var separator = null;
		var preview = '';
		while (lineNo < lines.length && lineNo < 5)
		{
			var line = lines[lineNo];
			if (lineNo < 4) 
			{
				preview += Strings.encodeTags(line) + '\n';
			}
			if (separator == null) separator = map.detectSeparator(line);
			if (mapFrom == null) 
			{
				if (line.match(/^[QCS]\d+([\t,;].*)?$/))// We look for lines starting with 'Q', 'C' or 'S' followed by {digits} and a {tab/comma/semicolon}
				{
					var firstChar = line.charAt(0);
					if (firstChar == 'Q')
					{
						mapFrom = 'PatientId';
					}
					else if (firstChar == 'C')
					{
						mapFrom = 'CaseId';
					}
					else if (firstChar == 'S')
					{
						mapFrom = 'SpecimenId';
					}
				}
				else if (lineNo == 0 && line.match(/^[^\t,;]*pad/i)) // If the first header column contains 'pad' (ignoring case) we assume it contains PAD values
				{
					mapFrom = 'PAD';
				}
			}
			lineNo++;
		}
		if (lines.length > 4) preview += '<div id="more-lines">+' + (lines.length-4) + ' more lines...</div>';
		frm.separator.value = separator ? separator.id : 'tab';
		frm.mapFrom.value = mapFrom || 'PersonalNo';
		Events.sendChangeEvent(frm.mapFrom);
		Doc.element('preview').innerHTML = preview;
	}

	/**
		Detect the "best" separator to use by selecting the one 
		that appear the most times in the given text line. If no
		separator is found null is returned.
	*/
	map.detectSeparator = function(text)
	{
		var separators =
		[
			{ 'id': 'tab', 'test': /\t/g, 'sep': '\t'},
			{ 'id': 'comma', 'test': /,/g, 'sep': ','},
			{ 'id': 'semicolon', 'test': /;/g, 'sep': ';'}
		];
		var maxCount = 0;
		var bestMatch = 0;
		for (var i=0; i < separators.length; i++)
		{
			var sep = separators[i];
			var m = text.match(sep.test);
			if (m && m.length > maxCount)
			{
				maxCount = m.length;
				bestMatch = i;
			}
		}
		return maxCount == 0 ? null : separators[bestMatch];
	}
	
	map.validateStep1 = function(event)
	{
		var valid = true;
		valid &= Wizard.isValid('file');
		if (!valid) event.preventDefault();
	}

	map.submit = function()
	{
		var frm = document.forms['reggie'];
		
		var formData = new FormData();
		var file = frm.file.files[0];
		formData.append('file', file, file.name);

		var url = '../Export.servlet?ID='+App.getSessionId();
		url += '&cmd=MapDataToReleases';
		url += '&separator='+frm.separator.value;
		url += '&mapFrom='+Strings.encodeTags(frm.mapFrom.value);
		url += '&exportCaseId='+(frm.exportCaseId.checked ? 1 : 0);
		url += '&exportSpecimenId='+(frm.exportSpecimenId.checked ? 1 : 0);
		url += '&exportPad='+(frm.exportPad.checked ? 1 : 0);
		url += '&sortOutput='+(frm.sortOutputOrder.checked ? 1 : 0);
		Wizard.showLoadingAnimation('Working...', 'map-data-progress');
		Wizard.asyncJsonRequest(url, map.submissionResults, 'POST', formData);
	}

	map.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('godownload');
		Doc.show('gorestart');
	}
	
	map.download = function()
	{
		var url = '../Export.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadMappedData';
		location.href = url;
	}

	return map;
}();

Doc.onLoad(MapData.initPage);

