var PersInfo = function()
{
	var persinfo = {};
	var debug = 0;
	
	var patientInfo = null;
	var caseInfo = null;
	var bloodInfo = null;

	var pnrIsTested = false;
	var pnrIsValid = false;
	var pnrIsAcceptable = false;
	var caseIsValid = false;

	var lateralityIsValid = false;
	var samplingDateIsValid = true;
	var rnaLaterDateIsValid = true;
	var copyConsentEnabled = false;

	persinfo.initPage = function()
	{
		
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', persinfo.validateStep1);
		Events.addEventHandler('caseName', 'keypress', Wizard.focusOnEnter);
		Events.addEventHandler('caseName', 'blur', persinfo.caseNameOnChange);
		Events.addEventHandler('personalNumber', 'keyup', persinfo.personalNumberOnChange);
		Events.addEventHandler('personalNumber', 'keypress', Wizard.goNextOnTabOrEnter);
		Events.addEventHandler('select-waiting-case', 'click', persinfo.showCaseSelection);
		Events.addEventHandler('select-case', 'click', persinfo.caseSelected);
		Events.addEventHandler(document.body, 'click', persinfo.hideCaseSelection);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', persinfo.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', persinfo.validateStep2);
		Events.addEventHandler('familyName', 'blur', persinfo.nameOnChange);
		Events.addEventHandler('allFirstNames', 'blur', persinfo.nameOnChange);
		Events.addEventHandler('allFirstNames', 'keypress', Wizard.goNextOnTabOrEnter);

		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', persinfo.initializeStep3);
		Events.addEventHandler('step-3', 'wizard-validate', persinfo.validateStep3);
		Events.addEventHandler('pad', 'blur', persinfo.padOnChange);
		Events.addEventHandler('externalSpecimenExists', 'change', persinfo.externalSpecimenExistsOnChange);
		Events.addEventHandler('samplingDate', 'blur', persinfo.samplingDateTimeOnChange);
		Events.addEventHandler('samplingTime', 'blur', persinfo.samplingDateTimeOnChange);
		Events.addEventHandler('samplingDate', 'keypress', persinfo.samplingDateTimeFocus);
		Events.addEventHandler('samplingTime', 'keypress', persinfo.samplingDateTimeFocus);
		Events.addEventHandler('rnaLaterDate', 'blur', persinfo.rnaLaterDateTimeOnChange);
		Events.addEventHandler('rnaLaterTime', 'blur', persinfo.rnaLaterDateTimeOnChange);
		Events.addEventHandler('noReferralForm', 'change', persinfo.noReferralFormOnChange);
		Buttons.addClickHandler('swapPad', persinfo.swapPad);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('goupdate', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', persinfo.submit);
		
		Doc.show('step-1');
		Doc.show('gonext');
		
		var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetSpecimenWithoutCase';
		Wizard.showLoadingAnimation('Loading information about specimen tubes...');
		Wizard.asyncJsonRequest(url, persinfo.onSpecimenTubesLoaded);
	}
	
	persinfo.onSpecimenTubesLoaded = function(response)
	{
		var specimenTubes = response.specimenTubes;
		if (specimenTubes.length == 0) return;
		
		var html = '';
		var lastPrefix = null;
		for (var spNo = 0; spNo < specimenTubes.length; spNo++)
		{
			var sp = specimenTubes[spNo];
			var prefix = sp.name.substr(0, 2);
			if (lastPrefix && prefix != lastPrefix)
			{
				html += '<div class="menuseparator"></div>';
			}
			lastPrefix = prefix;
			html += '<div class="menuitem interactable enabled" id="sp-'+sp.id+'" data-case-name="'+Strings.encodeTags(sp.name)+'">';
			html += Strings.encodeTags(sp.name);
			if (sp.numSpecimen > 1) html += ' ('+sp.numSpecimen+')';
			if (sp.externalRef) html += ' – '+Strings.encodeTags(sp.externalRef);
			html += '</div>';
		}
		Doc.element('select-case-all').innerHTML = html;
		Doc.element('num-waiting-cases').innerHTML = specimenTubes.length+(specimenTubes.length == 1 ? ' case':' cases')+' waiting for registration';
		Doc.show('waitingCasesSection');
	}
	
	persinfo.showCaseSelection = function(event)
	{
		var menu = Doc.element('select-case');
		menu.style.display = 'block';
		var x = event.clientX+1;
		var y = event.clientY;
		menu.style.left = (x)+'px';
		menu.style.top = (y)+'px';
		event.stopPropagation();
	}
	
	persinfo.hideCaseSelection = function()
	{
		Doc.hide('select-case');
	}
	
	persinfo.caseSelected = function(event)
	{
		var frm = document.forms['reggie'];
		var name = Data.get(event.target, 'case-name');
		frm.caseName.value = name;
		frm.personalNumber.focus();
		persinfo.caseNameOnChange();
	}

	
	persinfo.caseNameOnChange = function()
	{
		var frm = document.forms['reggie'];
		var caseName = frm.caseName.value.toUpperCase();
		caseIsValid = false;
		Doc.element('numSpecimenTubes').innerHTML = '';
		if (caseName == '')
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Missing');
			frm.caseName.focus();
			return;
		}
		if (!Reggie.isValidCaseName(caseName, true)) // Allow C/D suffix to be used
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Only 7-digits name + optional \'C\' or \'D\' is allowed.');
			frm.caseName.focus();
			return;
		}

		frm.caseName.value = caseName;
		Wizard.setInputStatus('caseName', 'valid');
		caseIsValid = true;
		
		var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo&caseName=' + encodeURIComponent(caseName);
		Wizard.showLoadingAnimation('Loading information about case...');
		Wizard.asyncJsonRequest(url, persinfo.onCaseInfoLoaded);
	}

	persinfo.onCaseInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		
		// Get biosource information from the AJAX response
		caseInfo = response.caseInfo;
		bloodInfo = response.bloodInfo;
		var numSpecimenTubes = caseInfo.specimen ? caseInfo.specimen.length : 0;
		Doc.element('numSpecimenTubes').innerHTML = '<i>('+(numSpecimenTubes || 'no')+' existing specimen '+(numSpecimenTubes==1?'item':'items')+')</i>';
		
		if (caseInfo && caseInfo.patient)
		{
			patientInfo = caseInfo.patient;
			frm.personalNumber.value = patientInfo.personalNumber;
			persinfo.personalNumberOnChange();
			pnrIsValid = true;
			Wizard.goNext(true);
		}
		else if (bloodInfo && bloodInfo.patient)
		{
			patientInfo = bloodInfo.patient;
			frm.personalNumber.value = patientInfo.personalNumber;
			persinfo.personalNumberOnChange();
			pnrIsValid = true;
			Wizard.goNext(true);
		}
	}

	persinfo.personalNumberOnChange = function()
	{
		var frm = document.forms['reggie'];
		var pnr = frm.personalNumber.value;
		pnrIsValid = false;
		pnrIsAcceptable = false;
		pnrIsTested = true;
		
		var pnrLen = pnr.length;
		if (pnrLen == 0)
		{
			Wizard.setInputStatus('personalNumber', 'invalid', 'A personal number is required');
			return;
		}
		
		pnrIsAcceptable = true;
		if (pnrLen != 12 && pnrLen != 10)
		{
			Wizard.setInputStatus('personalNumber', 'warning', pnrLen < 12 ? 'Too short' : 'Too long');
			return;
		}

		if (!pnr.match(/^\d+$/))
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Non-standard personal number');
			return;
		}
		
		var datePart = pnr.substring(0, pnrLen - 4); // Remove last four digits
		var dateFormat = pnrLen == 10 ? 'yyMMdd' : 'yyyyMMdd';
		var isValidDate = Dates.isDate(datePart, dateFormat);
		if (!isValidDate)
		{
			var dayInMonth = parseInt(datePart.substring(-2), 10); // Last two characters
			if (dayInMonth > 60)
			{
				// 'Samordningsnummer' has day-in-month + 60; check this
				dayInMonth -= 60;
				var tmpPnr = pnr.substring(0, pnrLen - 6); // First 4 or 6 digits is year+month
				if (dayInMonth < 10) tmpPnr += '0';
				tmpPnr += dayInMonth;
				isValidDate = Dates.isDate(tmpPnr, dateFormat);
			}
		}
		if (!isValidDate)
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Not a valid date');
			return;
		}
		
		if (!Reggie.personalNumberControlDigitCheck(pnr.substr(pnrLen == 10 ? 0 : 2)))
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Invalid control digit');
			return;
		}

		Wizard.setInputStatus('personalNumber', 'valid');
		pnrIsValid = true;
		Wizard.hideGoNextConfirmation();
	}

	persinfo.validateStep1 = function(event)
	{
		if (!pnrIsTested) persinfo.personalNumberOnChange();
		if (!caseIsValid || !pnrIsValid) 
		{
			if (!pnrIsAcceptable)
			{
				event.preventDefault();
				return;
			}
			var frm = document.forms['reggie'];
			if (!frm.verifyGoNext)
			{
				frm.personalNumber.focus();
				Wizard.showGoNextConfirmation(true, 'Check to verify registration of non-standard personal number.');
				event.preventDefault();
			}
		}
	}

	persinfo.initializeStep2 = function(event)
	{
		var frm = document.forms['reggie'];
		if (!patientInfo)
		{
			var pnr = frm.personalNumber.value;
			if (pnr.length == 10 && pnrIsValid) 
			{
				var year = Reggie.fullYear(parseInt(pnr.substr(0, 2), 10));
				pnr = year + pnr.substr(2);
			}
		
			var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
			url += '&cmd=GetPatientInfo';
			url += '&personalNumber=' + encodeURIComponent(pnr);
			url += '&pnrIsValid=' + pnrIsValid;
	
			Wizard.showLoadingAnimation('Loading patient information...');
			Wizard.asyncJsonRequest(url, persinfo.patientInfoLoaded);
		}
		else
		{
			persinfo.patientInfoLoaded();
		}
	}

	
	persinfo.patientInfoLoaded = function(response)
	{
		Wizard.setCurrentStep(2);
		// Get patient information from the AJAX response
		if (response) patientInfo = response.patientInfo;
		var frm = document.forms['reggie'];

		frm.patientCode.value=patientInfo.name;
		frm.familyName.value = patientInfo.familyName || '';
		frm.allFirstNames.value = patientInfo.allFirstNames || '';
		Doc.element('dateOfBirth').innerHTML = patientInfo.dateOfBirth ? Strings.encodeTags(patientInfo.dateOfBirth) : '---';
		Doc.element('gender').innerHTML = patientInfo.gender ? Strings.encodeTags(patientInfo.gender) : '---';
		frm.familyName.focus();

		if (!patientInfo.id)
		{
			if (!pnrIsValid)
			{
				Wizard.setInputStatus('gender', 'warning', 'Gender and DateOfBirth can\'t be automatically generated for non-standard Personal number');
			}
		}
		else
		{
			Doc.element('step-2-title').innerHTML = 'Existing patient: Verify names';
			frm.patientCode.disabled = true;
		}
		
		Doc.show('gonext');
		Doc.show('gocancel');
		
	}
	
	persinfo.nameOnChange = function(event)
	{
		var field = event.target;
		if (field.value == '')
		{
			Wizard.setInputStatus(field, 'warning', 'Missing');
		}
		else
		{
			Wizard.setInputStatus(field, 'valid');
			Wizard.hideGoNextConfirmation();
		}
	}

	persinfo.validateStep2 = function(event)
	{
		var valid = true;
		var acceptable = true;
		var frm = document.forms['reggie'];

		if (frm.allFirstNames.value == '')
		{
			Wizard.setInputStatus('allFirstNames', 'warning', 'Missing');
			valid = false;
		}
		else
		{
			Wizard.setInputStatus('allFirstNames', 'valid');
		}
		patientInfo.allFirstNames = frm.allFirstNames.value;
		
		if (frm.familyName.value == '')
		{
			Wizard.setInputStatus('familyName', 'warning', 'Missing');
			valid = false;
		}
		else
		{
			Wizard.setInputStatus('familyName', 'valid');
		}
		patientInfo.familyName = frm.familyName.value;

		// New patient only
		if (!patientInfo.id)
		{
			if (frm.patientCode.value == '')
			{
				Wizard.setInputStatus('patientCode', 'invalid', 'Missing');
				frm.patientCode.focus();
				valid = false;
				acceptable = false;
			}
			else
			{
				Wizard.setInputStatus('patientCode', 'valid');
				patientInfo.name = frm.patientCode.value;
			}	
		}
		
		if (event.detail.auto)
		{
			if (!valid)
			{
				Wizard.showGoNextConfirmation(true, 'Check to verify registration with missing name(s).');
				event.preventDefault();
			}
		}
		else
		{
			if (!acceptable) event.preventDefault();
		}
		
	}
	
	persinfo.initializeStep3 = function()
	{
		// Check entered case and pnr with AJAX
		var frm = document.forms['reggie'];
		var updateMode = caseInfo.id && caseInfo.patient;
		var isPreCase = caseInfo.id && !caseInfo.patient && caseInfo.consent; // The case is a pre-registered consent
		
		// Generate list of specimen tubes
		var hasLeftSpecimen = false;
		var hasRightSpecimen = false;
		var hasUnknownSpecimen = false;
		var thisCaseLaterality = caseInfo.laterality;
		var hasSpecimen = caseInfo.specimen && caseInfo.specimen.length > 0;
		var thisCasePAD = null;
		var thisCaseAlternatePAD = null;
		var thisCaseExternalOperator = null;
		var hasPAD = false;
		var thisCaseSamplingDate = null;
		var thisCaseRNALaterDate = null;
		
		if (hasSpecimen)
		{
			Doc.element('specimenTubes').innerHTML = persinfo.createSpecimenHtml();
			
			var downloadable = Doc.element('specimenTubes').getElementsByClassName('download-file');
			for (var fileNo = 0; fileNo < downloadable.length; fileNo++)
			{
				Events.addEventHandler(downloadable[fileNo], 'click', persinfo.downloadJsonFile);
			}
			
			for (var i = 0; i < caseInfo.specimen.length; i++)
			{
				var specimen = caseInfo.specimen[i];
				var isYellow = specimen.YellowLabel != null;
				specimen.YellowLabelOriginal = specimen.YellowLabel;
	
				frm['yellow.'+specimen.id].checked = isYellow;
				Forms.selectListOption(frm['specimenType.'+specimen.id], specimen.specimenType);
				Forms.selectListOption(frm['biopsyType.'+specimen.id], specimen.biopsyType);
				frm['comment.'+specimen.id].value = specimen.operatorDeliveryComment;
				frm['otherPathNote.'+specimen.id].value = specimen.otherPathNote;
				Events.addEventHandler('yellow.'+specimen.id, 'click', persinfo.yellowOnChange);

				if (specimen.pad && !hasPAD) 
				{
					hasPAD = true;
					thisCasePAD = specimen.pad;
				}
				if (specimen.alternatePad && thisCaseAlternatePAD == null)
				{
					thisCaseAlternatePAD = specimen.alternatePad;
				}
				if (specimen.externalOperator && thisCaseExternalOperator == null)
				{
					thisCaseExternalOperator = specimen.externalOperator;
				}
				if (specimen.samplingDate && !thisCaseSamplingDate) 
				{
					thisCaseSamplingDate = specimen.samplingDate;
				}
				if (specimen.rnaLaterDate && !thisCaseRNALaterDate) 
				{
					thisCaseRNALaterDate = specimen.rnaLaterDate;
				}
				if (specimen.laterality) 
				{
					Forms.checkRadio(frm.laterality, specimen.laterality);
					if (specimen.laterality == 'LEFT') hasLeftSpecimen = true;
					if (specimen.laterality == 'RIGHT') hasRightSpecimen = true;
					if (!thisCaseLaterality) thisCaseLaterality = specimen.laterality;
				}
				else
				{
					hasUnknownSpecimen = true;
				}
			}
			
			// Check that all specimen tubes have the same laterality
			var numLateralities = 0;
			if (hasLeftSpecimen) numLateralities++;
			if (hasRightSpecimen) numLateralities++;
			if (hasUnknownSpecimen) numLateralities++;
			if (numLateralities > 1)
			{
				Wizard.setInputStatus('laterality', 'warning', 'Specimen tubes with different laterality');
				thisCaseLaterality = null;
			}
			
			if (thisCaseExternalOperator != null || thisCaseAlternatePAD != null)
			{
				Doc.show('externalSpecimenExistsSection');
				Doc.show('externalOperatorSection');
				Doc.show('alternatePadSection');
			}
			
			Doc.show('samplingDateSection');
			Doc.show('rnaLaterDateSection');
			Doc.show('specimenTubes');
		}
		else
		{
			if (caseInfo.noSpecimen)
			{
				var noSpecimen = caseInfo.noSpecimen;
				// This case is registered with 'NoSpecimen'
				thisCasePAD = noSpecimen.pad;
				thisCaseAlternatePAD = noSpecimen.alternatePad;
				thisCaseExternalOperator = noSpecimen.externalOperator;
				thisCaseSamplingDate = noSpecimen.samplingDate;
				thisCaseRNALaterDate = noSpecimen.rnaLaterDate;
				frm.reasonIfNoSpecimen.value = noSpecimen.reasonIfNoSpecimen;
				Doc.element('noSpecimenTubes').innerHTML = '<i>Not found – NoSpecimen item already exists and can be updated</i>';
				Doc.element('existingItemName').innerHTML = Strings.encodeTags(noSpecimen.name);
				Doc.show('existingItemNameSection');
				
				frm.externalSpecimenExists.checked = noSpecimen.externalSpecimenExists == 'Yes';
				frm.otherPathNote.value = noSpecimen.otherPathNote;
				frm.liasonComment.value = noSpecimen.liasonComment;
			}
			else
			{
				Doc.element('noSpecimenTubes').innerHTML = '<i>Not found – NoSpecimen item will be created</i>';
				Forms.selectListOption(frm.externalOperator, 'CMD');
			}
			Doc.show('noSpecimenSection');
			persinfo.externalSpecimenExistsOnChange();
		}

		if (thisCaseExternalOperator)
		{
			Forms.selectListOption(frm.externalOperator, thisCaseExternalOperator);
			if (frm.externalOperator.selectedIndex == 0)
			{
				frm.externalOperator[frm.externalOperator.length] = new Option(thisCaseExternalOperator, thisCaseExternalOperator, true, true);
			}
		}

		if (caseInfo.coreBiopsies && caseInfo.coreBiopsies.length)
		{
			var html = '<i>';
			for (var i = 0; i < caseInfo.coreBiopsies.length; i++)
			{
				var cb = caseInfo.coreBiopsies[i];
				html += Strings.encodeTags(cb.name+' ('+cb.laterality) + ')<br>';
			}
			html += '</i>';
			Doc.element('coreBiopsies').innerHTML = html;
			Doc.show('coreBiopsiesSection');
		}
		
		// Populate 'Sampling date' and 'RNA Later date' fields
		if (thisCaseSamplingDate != null)
		{
			frm.samplingDate.value = thisCaseSamplingDate.substring(0, 8);
			frm.samplingTime.value = thisCaseSamplingDate.substring(9, 13);
		}
		persinfo.samplingDateTimeOnChange();
		if (thisCaseRNALaterDate != null)
		{
			frm.rnaLaterDate.value = thisCaseRNALaterDate.substring(0, 8);
			frm.rnaLaterTime.value = thisCaseRNALaterDate.substring(9, 13);
		}
		persinfo.rnaLaterDateTimeOnChange();
		
		frm.pad.value = thisCasePAD;
		frm.alternatePad.value = thisCaseAlternatePAD;
		
		Doc.show('caseSection');
		
		// Check the blood registration for consent information
		if (!isPreCase && !copyConsentEnabled && bloodInfo && bloodInfo.consent)
		{
			var html = "From blood '" + Strings.encodeTags(bloodInfo.name);
			html += "' (" + bloodInfo.consent;
			if (bloodInfo.consentDate) html += '; ' + Reggie.reformatDate(bloodInfo.consentDate);
			html += ')';
			Doc.element('copyConsent.text').innerHTML = html;
			frm.copyConsent.value = bloodInfo.id;
			copyConsentEnabled = true;
		}
		
		// Existing cases for this patient
		var hasLeftCase = false;
		var hasRightCase = false;
		var hasUnknownCase = false;
		if (patientInfo.cases && patientInfo.cases.length > 0)
		{
			var cases = '';
			for (var i = 0; i < patientInfo.cases.length; i++)
			{
				var cc = patientInfo.cases[i];
				if (cc.laterality == 'LEFT') 
				{
					if (hasLeftCase && !updateMode) 
					{
						Wizard.setInputStatus('laterality', 'warning', 'Two cases with laterality=LEFT');
					}
					hasLeftCase = true;
				}
				else if (cc.laterality == 'RIGHT') 
				{
					if (hasRightCase && !updateMode) 
					{
						Wizard.setInputStatus('laterality', 'warning', 'Two cases with laterality=RIGHT');
					}
					hasRightCase = true;
				}
				else
				{
					hasUnknownCase = true;	
				}
				if (!updateMode || cc.id == caseInfo.id)
				{
					cases += '<label><input type="radio" name="laterality" value="' + (updateMode ? cc.laterality || cc.id : cc.id) + '" id="laterality'+cc.id+'"';
					if (cc.laterality == thisCaseLaterality) cases += ' checked';
					cases += '>';
					if (cc.laterality) 
					{
						cases += cc.laterality;
					}
					else
					{
						cases += '<i>unknown laterality</i>';
					}
					cases += '</label>';
					if (!updateMode)
					{
						cases += ' [merge with ' + Strings.encodeTags(cc.name) + ']';
					}
					cases += '<br>';
				}
			}
			
			// Create the 'new case' option (which may also be an existing pre-registered consent)
			var newCaseText = isPreCase ? ' [<i>pre-registered consent: ' + persinfo.toConsentString(caseInfo) + '</i>]' : ' [<i>new case</i>]';
			if (!hasLeftCase)
			{
				cases += '<label><input type="radio" name="laterality" value="LEFT" id="lateralityLeft"';
				if (thisCaseLaterality == 'LEFT') cases += ' checked';
				cases += '>LEFT</label>';
				if (!updateMode) cases += newCaseText;
				cases += '<br>';
			}
			if (!hasRightCase)
			{
				cases += '<label><input type="radio" name="laterality" value="RIGHT" id="lateralityRight"';
				if (thisCaseLaterality == 'RIGHT') cases += ' checked';
				cases += '>RIGHT</label>';
				if (!updateMode) cases += newCaseText;
				cases += '<br>';
			}
			if (!updateMode && !hasLeftCase && !hasRightCase)
			{
				cases += '<label><input type="radio" name="laterality" value="" id="lateralityUnknown"';
				if (thisCaseLaterality == null) cases += ' checked';
				cases += '><i>unknown laterality</i></label>' + newCaseText;
				cases += '<br>'
			}
			
			if (!isPreCase && !updateMode && !copyConsentEnabled && patientInfo.cases[0].consent)
			{
				var c = patientInfo.cases[0];
				var html = "From case '" + Strings.encodeTags(c.name);
				html += "' (" + persinfo.toConsentString(c) + ')';
				Doc.element('copyConsent.text').innerHTML = html;
				frm.copyConsent.value = patientInfo.cases[0].id;
				copyConsentEnabled = true;
			}
			
			if (updateMode)
			{
				if (caseInfo.merged)
				{
					Doc.element('step-3-title').innerHTML = 'Update case \''+Strings.encodeTags(caseInfo.originalName)+'\' (which is merged with case \'' + Strings.encodeTags(caseInfo.name) + '\')';
				}
				else
				{
					Doc.element('step-3-title').innerHTML = 'Update case \''+Strings.encodeTags(caseInfo.name)+'\'';
				}
				
			}
			else if (patientInfo.cases.length == 2)
			{
				Doc.element('step-3-title').innerHTML = 'Merge with existing case';	
			}
			else if (patientInfo.cases.length > 0)
			{
				Doc.element('step-3-title').innerHTML = 'Create new case or merge with existing case';
			}
			
			Doc.element('laterality.input').innerHTML = cases;
		}
		
		for (var i = 0; i < frm.laterality.length; i++)
		{
			Events.addEventHandler(frm.laterality[i], 'click', persinfo.lateralityOnChange);
		}
		persinfo.lateralityOnChange();

		Wizard.setCurrentStep(3);
		if (updateMode)
		{
			Doc.show('goupdate');
		}
		else
		{
			Doc.show('goregister');
		}
		Doc.show('gocancel');
		if (lateralityIsValid) frm.pad.focus();
	}
	
	persinfo.externalSpecimenExistsOnChange = function()
	{
		var frm = document.forms['reggie'];
		persinfo.externalSpecimenExists(frm.externalSpecimenExists.checked);
		frm.samplingDate.focus();
	}
	
	persinfo.externalSpecimenExists = function(enabled)
	{
		var frm = document.forms['reggie'];
		if (enabled)
		{
			Doc.show('samplingTimeField', 'inline');
			frm.samplingTime.disabled = false;
			Doc.show('rnaLaterDateSection');
			Doc.hide('reasonIfNoSpecimenSection');
			Doc.show('otherPathNoteSection');
			Doc.show('externalOperatorSection');
			Doc.show('alternatePadSection');
			Doc.show('liasonCommentSection');
		}
		else
		{
			Doc.hide('samplingTimeField');
			frm.samplingTime.disabled = true;
			Doc.hide('rnaLaterDateSection');
			Doc.show('reasonIfNoSpecimenSection');
			Doc.hide('otherPathNoteSection');
			Doc.hide('externalOperatorSection');
			Doc.hide('alternatePadSection');
			Doc.hide('liasonCommentSection');
		}
	}
	
	persinfo.downloadJsonFile = function(event)
	{
		var fileId = Data.get(event.currentTarget, 'file-id');
		var fileName = Data.get(event.currentTarget, 'file-name');

		var url = '../ExternalSpecimen.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadJsonFile';
		url += '&fileId='+fileId;
		
		Dialogs.openPopup(url, fileName, 800, 800);
	}

	
	persinfo.createSpecimenHtml = function()
	{
		var html = '';
		for (var i = 0; i < caseInfo.specimen.length; i++)
		{
			var specimen = caseInfo.specimen[i];
			var isYellow = specimen.YellowLabel != null;
			var isPaused = specimen.bioWell && specimen.bioWell.bioPlate.name.indexOf('PSp') == 0;
			specimen.YellowLabelOriginal = specimen.YellowLabel;

			html += '<tr class="section-header">';
			html += '<td colspan="4">Specimen Tube '+Strings.encodeTags(specimen.name)+'</td>';
			html += '</tr>';
			
			html += '<tr>';
			html += '<td class="subprompt">Laterality</td>';
			html += '<td class="input">'+Strings.encodeTags(specimen.laterality || 'unknown')+'</td>';
			html += '<td class="status" id="laterality.'+specimen.id+'.status"></td>';
			html += '<td class="help"><span id="laterality.'+specimen.id+'.message" class="message"></span></td>';		
			html += '</tr>';
			
			if (specimen.externalRef)
			{
				html += '<tr>';
				html += '<td class="subprompt">External reference</td>';
				html += '<td class="input" colspan="2" style="background-color: #f0f080; padding: 4px;">';
				html += Strings.encodeTags(specimen.externalRef);
				if (specimen.importedFrom)
				{
					var file = specimen.importedFrom;
					html += ' <span class="link download-file" data-file-id="'+file.id+'" data-file-name="'+file.name+'"' +
						' title="View the raw data from the JSON file that was imported"><img src="../images/download.png"></span>';
				}
				html += '</td>';
				html += '<td class="help">The JSON file that was imported to this specimen.</td>';		
				html += '</tr>';
			}

			html += '<tr id="specimen-row.'+specimen.id+'" class="specimen-tube'+(isYellow?' yellow':'')+(isPaused?' paused':'')+'">';
			html += '<td class="subprompt yellow-tag"><label for="yellow.'+specimen.id+'">Yellow label</label></td>';
			html += '<td class="input">';
			html += '<input type="checkbox" name="yellow.'+specimen.id+'" id="yellow.'+specimen.id+'"';
			html += ' data-specimen-id="'+specimen.id+'"';
			html += '></td>';
			html += '<td class="status"></td>';
			html += '<td class="help"></td>';		
			html += '</tr>';

			html += '<tr>'
			html += '<td class="subprompt">Biopsy type</td>';
			html += '<td class="input">';
			html += '<select name="biopsyType.'+specimen.id+'" id="biopsyType.'+specimen.id+'">';
			html += '<option selected value="">unknown';
			html += '<option value="SpecimenSurgery">SpecimenSurgery';
			html += '<option value="SpecimenCoreBiopsy">SpecimenCoreBiopsy';
			html += '<option value="SpecimenCoreBiopsy2nd">SpecimenCoreBiopsy2nd';
			html += '<option value="SpecimenFineNeedleAspiration">SpecimenFineNeedleAspiration';
			html += '</select>';
			html += '</td>';
			html += '<td class="status" id="biopsyType.'+specimen.id+'.status"></td>';
			html += '<td class="help"><span id="biopsyType.'+specimen.id+'.message" class="message"></span></td>';		
			html += '</tr>';

			
			html += '<tr>'
			html += '<td class="subprompt">Specimen type</td>';
			html += '<td class="input">';
			html += '<select name="specimenType.'+specimen.id+'" id="specimenType.'+specimen.id+'">';
			html += '<option selected value="">unknown';
			html += '<option value="Primary">Primary';
			html += '<option value="LymphnodeRegional">LymphnodeRegional';				
			html += '<option value="MetastasisDistant">MetastasisDistant';
			html += '<option value="RecurrenceLocal">RecurrenceLocal';
			html += '<option value="RecurrenceRegional">RecurrenceRegional';
			html += '<option value="Normal">Normal';
			html += '<option value="Uncertain">Uncertain';
			html += '</select>';
			html += '</td>';
			html += '<td class="status" id="specimenType.'+specimen.id+'.status"></td>';
			html += '<td class="help"><span id="specimenType.'+specimen.id+'.message" class="message"></span></td>';		
			html += '</tr>';

			html += '<tr class="align-top">';
			html += '<td class="subprompt">Other path note</td>';
			html += '<td class="input"><textarea rows="3" name="otherPathNote.'+specimen.id+'" id="otherPathNote.'+specimen.id+'"></textarea></td>';
			html += '<td class="status" id="otherPathNote.'+specimen.id+'.status"></td>';
			html += '<td class="help"><span id="otherPathNote.'+specimen.id+'.message" class="message"></span>In this field, enter ALL text noted on the referral form by the pathologist.</td>';		
			html += '</tr>';
			
			html += '<tr class="align-top">';
			html += '<td class="subprompt">Operator delivery comment</td>';
			html += '<td class="input"><textarea rows="3" name="comment.'+specimen.id+'" id="comment.'+specimen.id+'"></textarea></td>';
			html += '<td class="status" id="comment.'+specimen.id+'.status"></td>';
			html += '<td class="help"><span id="comment.'+specimen.id+'.message" class="message"></span>In this field, enter information specific for the specimen tube.</td>';		
			html += '</tr>';
		}
		return html;
	}
	
	persinfo.toConsentString = function(item)
	{
		var s = item.consent;
		if (item.consentDate) 
		{
			s += '; ' + Reggie.reformatDate(item.consentDate);
		}
		return s;
	}
	
	persinfo.yellowOnChange = function(event)
	{
		var spId = Data.get(event.currentTarget, 'specimen-id');
		Doc.addOrRemoveClass('specimen-row.'+spId, 'yellow', event.currentTarget.checked);
	}
	
	persinfo.lateralityOnChange = function()
	{
		lateralityIsValid = false;
		var updateMode = caseInfo.id;
		
		// Check selected laterality against specimen tubes
		var frm = document.forms['reggie'];
		var selectedLaterality = Forms.getCheckedRadio(frm.laterality);
		
		// No laterality/case selected
		if (selectedLaterality == null) 
		{
			Wizard.setInputStatus('laterality', 'invalid', 'Not selected');
			return;
		}
		
		var numCases = patientInfo.cases ? patientInfo.cases.length : 0;
		Wizard.setInputStatus('laterality', 'valid', updateMode && numCases > 1 ? 'NOTE! Some alternatives may be missing if there is another case for the patient.' : '');
		lateralityIsValid = true;
			
		var laterality = selectedLaterality.value;
		var mergeWithCase = laterality.match(/\d+/);
		if (mergeWithCase)
		{
			// 'laterality' is the ID of an existing case
			for (var i = 0; i < patientInfo.cases.length; i++)
			{
				var cse = patientInfo.cases[i];
				if (cse.id == laterality)
				{
					laterality = cse.laterality;
					break;
				}
			}
		}
		else if (laterality == '') 
		{
			laterality = null;
		}
		
		// If the selected laterality/case is a new case it is possible to copy consent
		Doc.showHide('copyConsentSection', copyConsentEnabled && !mergeWithCase);
			
		// No specimen tubes?
		if (caseInfo.specimen)
		{
			for (var i = 0; i < caseInfo.specimen.length; i++)
			{
				var specimen = caseInfo.specimen[i];
				if (specimen.laterality != laterality)
				{
					Wizard.setInputStatus('laterality', 'valid', 'Specimen tubes are updated to: ' + laterality)
					return;
				}
			}
		}
		
		if (caseInfo.coreBiopsies)
		{
			for (var i = 0; i < caseInfo.coreBiopsies.length; i++)
			{
				var cb = caseInfo.coreBiopsies[i];
				if (cb.laterality != laterality)
				{
					Wizard.setInputStatus('laterality', 'valid', 'Core biopsies are updated to: ' + laterality);
					return;
				}
			}
		}
		
	}

	persinfo.padOnChange = function()
	{
		var frm = document.forms['reggie'];
		var PAD = frm.pad.value;
		if (PAD == '')
		{
			Wizard.setInputStatus('pad', 'warning', 'Missing');
			return;
		}
		frm.pad.value = PAD.replace(/\+/, '-');
		Wizard.setInputStatus('pad', 'valid');
	}

	persinfo.samplingDateTimeFocus = function(event)
	{
		if (event.keyCode == 13)
		{
			var focusTo = caseInfo.specimen && caseInfo.specimen.length > 0 ? 'rnaLaterDate' : 'reasonIfNoSpecimen'
			Doc.element(focusTo).focus();
		}
	}
	
	persinfo.samplingDateTimeOnChange = function()
	{
		var frm = document.forms['reggie'];
		samplingDateIsValid = false;
		Wizard.setInputStatus('samplingDate');
		
		var samplingDate = frm.samplingDate.value;
		var samplingTime = frm.samplingTime.disabled ? null : frm.samplingTime.value;
		
		if (samplingDate == '')
		{
			Wizard.setInputStatus('samplingDate', 'warning', 'Missing date');
		}
		else
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			samplingDate = Reggie.autoFillDate(samplingDate, true);
			frm.samplingDate.value = samplingDate;

			if (!frm.samplingTime.disabled)
			{
				samplingTime = Reggie.autoFillTime(samplingTime);
				frm.samplingTime.value = samplingTime;
			}
			
			if (samplingDate != '' && !Dates.isDate(samplingDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('samplingDate', 'invalid', 'Not a valid date');
				return;
			}
			if (frm.rnaLaterDate.value == '') frm.rnaLaterDate.value = samplingDate;
			
			if (!frm.samplingTime.disabled)
			{
				if (!Dates.isDate(samplingDate + ' ' + samplingTime, 'yyyyMMdd HHmm'))
				{
					if (samplingDate != '') 
					{
						Wizard.setInputStatus('samplingDate', 'invalid', 'Not a valid time');
					}
					return;
				}
			}
			Wizard.setInputStatus('samplingDate', 'valid');
		}
		samplingDateIsValid = true;
	}

	persinfo.rnaLaterDateTimeOnChange = function()
	{
		var frm = document.forms['reggie'];
		rnaLaterDateIsValid = false;
		Wizard.setInputStatus('rnaLaterDate');
		
		var rnaLaterDate = frm.rnaLaterDate.value;
		var rnaLaterTime = frm.rnaLaterTime.value;
		var rnaLaterTimestamp;
		
		if (rnaLaterDate != '' || rnaLaterTime != '')
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			rnaLaterDate = Reggie.autoFillDate(rnaLaterDate, true);
			frm.rnaLaterDate.value = rnaLaterDate;

			rnaLaterTime = Reggie.autoFillTime(rnaLaterTime);
			frm.rnaLaterTime.value = rnaLaterTime;
			
			if (!Dates.isDate(rnaLaterDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('rnaLaterDate', 'invalid', 'Not a valid date');
				return;
			}
			rnaLaterTimestamp = Dates.parseString(rnaLaterDate + ' ' + rnaLaterTime, 'yyyyMMdd HHmm');
			if (rnaLaterTimestamp == null)
			{
				if (rnaLaterDate != '') 
				{
					Wizard.setInputStatus('rnaLaterDate', 'invalid', 'Not a valid time');
				}
				return;
			}
			Wizard.setInputStatus('rnaLaterDate', 'valid');
		}
		
		rnaLaterDateIsValid = true;

		if (samplingDateIsValid && rnaLaterTimestamp && !frm.samplingTime.disabled)
		{
			var samplingTimestamp = Dates.parseString(frm.samplingDate.value + ' ' + frm.samplingTime.value, 'yyyyMMdd Hmm');
			if (samplingTimestamp)
			{
				if (rnaLaterTimestamp.getDate() != samplingTimestamp.getDate())
				{
					Wizard.setInputStatus('rnaLaterDate', 'warning', 'Sampling and RNA later dates are different');
				}
				else if (rnaLaterTimestamp.getTime() < samplingTimestamp.getTime())
				{
					Wizard.setInputStatus('rnaLaterDate', 'warning', 'RNA later time is before Sampling time');
				}
			}
		}
	}

	persinfo.noReferralFormOnChange = function()
	{
		var frm = document.forms['reggie'];
		if (frm.noReferralForm.checked)
		{
			frm.reasonIfNoSpecimen.disabled = true;
			frm.reasonIfNoSpecimen.oldValue = frm.reasonIfNoSpecimen.value;
			frm.reasonIfNoSpecimen.value = 'no_referral_form';
		}
		else
		{
			frm.reasonIfNoSpecimen.disabled = false;
			frm.reasonIfNoSpecimen.value = frm.reasonIfNoSpecimen.oldValue || '';
		}
	}
	
	persinfo.swapPad = function()
	{
		var frm = document.forms['reggie'];
		var pad = frm.pad.value;
		var alternatePad = frm.alternatePad.value;
		frm.pad.value = alternatePad;
		frm.alternatePad.value = pad;
		persinfo.padOnChange();
	}
	
	persinfo.validateStep3 = function(event)
	{
		var frm = document.forms['reggie'];
		var isValid = lateralityIsValid && samplingDateIsValid;
		if (caseInfo.specimen && caseInfo.specimen.length > 0)
		{
			isValid = isValid && rnaLaterDateIsValid;
		}
		if (frm.externalSpecimenExists.checked)
		{
			isValid = isValid && rnaLaterDateIsValid;
		}
		if (!isValid) event.preventDefault();
	}

	persinfo.submit = function()
	{
		var frm = document.forms['reggie'];

		caseInfo.laterality = Forms.getCheckedRadio(frm.laterality).value;
		caseInfo.reasonIfNoSpecimen = frm.reasonIfNoSpecimen.value;
		caseInfo.pad = frm.pad.value;
		caseInfo.alternatePad = frm.alternatePad.value;
		
		var caseName = frm.caseName.value;
		var suffix = caseName.length > 7 ? caseName.substring(7, 8) : null;
		if (suffix != null)
		{
			if (suffix == 'C')
			{
				// 'C' suffix is only for 'SpecimenCoreBiopsy' items
				caseInfo.biopsyType = 'SpecimenCoreBiopsy';
			}
			if (suffix == 'D')
			{
				// 'D' suffix is only for 'SpecimenCoreBiopsy2nd' items
				caseInfo.biopsyType = 'SpecimenCoreBiopsy2nd';
			}
		}
		caseInfo.externalSpecimenExists = frm.externalSpecimenExists.checked;
		caseInfo.externalOperator = frm.externalOperator.value;
		caseInfo.liasonComment = frm.liasonComment.value;
		caseInfo.samplingDate = Strings.trim(frm.samplingDate.value + ' ' + frm.samplingTime.value);
		caseInfo.rnaLaterDate = Strings.trim(frm.rnaLaterDate.value + ' ' + frm.rnaLaterTime.value);
		caseInfo.otherPathNote = frm.otherPathNote.value;
		if (frm.copyConsent.checked && frm.copyConsent.value)
		{
			caseInfo.copyConsent = parseInt(frm.copyConsent.value, 10);
		}

		if (caseInfo.specimen && caseInfo.specimen.length > 0)
		{
			for (var i = 0; i < caseInfo.specimen.length; i++)
			{
				var specimen = caseInfo.specimen[i];
				specimen.YellowLabel = frm['yellow.'+specimen.id].checked ? 'yellow' : null;
				specimen.specimenType = frm['specimenType.'+specimen.id].value;
				specimen.biopsyType = frm['biopsyType.'+specimen.id].value;
				specimen.otherPathNote = frm['otherPathNote.'+specimen.id].value;
				specimen.operatorDeliveryComment = frm['comment.'+specimen.id].value;
			}
		}

		var updateMode = caseInfo.id && caseInfo.patient;
		var submitInfo = {};
		submitInfo.patientInfo = patientInfo;
		submitInfo.caseInfo = caseInfo;
		
		var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=' + (updateMode ? 'UpdateCase' : 'CreateCase');
		Wizard.showLoadingAnimation('Registering case...');
		Wizard.asyncJsonRequest(url, persinfo.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	persinfo.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return persinfo;
}();

Doc.onLoad(PersInfo.initPage);

