var StorePaused = function()
{
	var store = {};
	var debug = 0;
	
	var specimenTubes;
	
	// Page initialization
	store.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', store.validateStep1);
		Buttons.addClickHandler('downloadLabelsCsv', store.downloadLabels);
		Buttons.addClickHandler('downloadLabelsXlsx', store.downloadLabels);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('goprint', store.printPickList);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', store.submit);

		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetSpecimenTubesToStore';
		Wizard.showLoadingAnimation('Loading specimen tube information...');
		Wizard.asyncJsonRequest(url, store.initializeStep1);
	}
	
	store.initializeStep1 = function(response)
	{
		specimenTubes = response.specimenTubes;
		
		if (!specimenTubes || specimenTubes.length == 0)
		{
			Wizard.setFatalError('No specimen tubes available for storing.');
			return;
		}
		
		var html = '<table id="specimenTable">';
		html += '<thead class="bg-filled-100">';
		// First header row
		html += '<tr>';
		html += '<th>Specimen</th>';
		html += '<th>Arrival date</th>';
		html += '<th>Storage box</th>';
		html += '<th>Freezer</th>';
		html += '<th>Remark</th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		
		var homeUrl = Data.get('page-data', 'home-url');
		var yellowImg = '<img src="'+homeUrl+'/images/yellow-label.png">';
		for (var i=0; i < specimenTubes.length; i++)
		{
			var specimen = specimenTubes[i];
			var well = specimen.bioWell;
			var plate = well.bioPlate;
			var storage = plate.storage;
			var isYellow = specimen.specimen && specimen.specimen.YellowLabel != null;
			
			var freezerLocation = [];
			if (storage.section) freezerLocation[freezerLocation.length] = 'Section ' + storage.section;
			if (storage.tray) freezerLocation[freezerLocation.length] = 'Tray ' + storage.tray;
			if (storage.position) freezerLocation[freezerLocation.length] = 'Position ' + storage.position;
			

			var img = isYellow ? yellowImg : '';
			html += '<tr class="highlight'+(isYellow ? ' yellow-specimen' : '')+'">';
			html += '<td class="specimen if-yellow">'+img+Strings.encodeTags(specimen.name)+'</td>';
			html += '<td class="arrivaldate">'+Reggie.reformatDate(specimen.ArrivalDate)+'</td>';
			html += '<td class="box">'+Strings.encodeTags(plate.name+' '+well.location)+'</td>';
			html += '<td class="location">'+Strings.encodeTags(storage.name)+'<br>'+Strings.encodeTags(freezerLocation.join(', '))+'</td>';
			html += '<td class="remarks"></td>';
			html += '</tr>'
		}
		html += '</tbody>';
		html += '</table>';
		
		Doc.element('specimenList').innerHTML = html;

		Doc.show('step-1');
		Doc.show('goregister');
		Doc.show('goprint');
		Wizard.showGoNextConfirmation(true, 'If needed, print the protocol before registering!');
	}
	

	store.downloadLabels = function(event)
	{
		var format = Data.get(event.currentTarget, 'format');
		var specimen = [];
		for (var i = 0; i < specimenTubes.length; i++)
		{
			specimen[specimen.length] = specimenTubes[i].id;
		}
		
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetSpecimenTubeLabels';
		url += '&specimen=' + encodeURIComponent(specimen.join(','));
		url += '&format='+encodeURIComponent(format);
		window.open(url);
	}
	
	
	store.printPickList = function()
	{
		Wizard.hideGoNextConfirmation();
		Doc.removeClass('goregister', 'disabled');
		var reportName = 'Pick-list protocol for lysate re-extraction';
		var printNote = '<b>Note!</b> For better printing set page orientation to <i>portrait</i>.';
		Reggie.openPrintWindow('full-protocol', reportName, 'portrait', printNote, '../', 'store_paused.css');
	}

	store.submit = function()
	{
		var specimen = [];
		for (var i = 0; i < specimenTubes.length; i++)
		{
			specimen[specimen.length] = specimenTubes[i].id;
		}
		
		var submitInfo = {};
		submitInfo.specimen = specimen;
		
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterPausedSpecimen';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, store.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	store.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return store;
}();

Doc.onLoad(StorePaused.initPage);

